import unittest

import odil

class TestUIDsDictionaryEntry(unittest.TestCase):
    def test_constructor(self):
        entry = odil.UIDsDictionaryEntry(
            "MR Image Storage", "MRImageStorage",  "SOP Class")
        self.assertEqual(entry.name, "MR Image Storage")
        self.assertEqual(entry.keyword, "MRImageStorage")
        self.assertEqual(entry.type, "SOP Class")

class TestUIDsDictionary(unittest.TestCase):
    def test_get_item(self):
        entry = odil.registry.uids_dictionary["1.2.840.10008.5.1.4.1.1.4"]
        self.assertEqual(entry.name, "MR Image Storage")
        self.assertEqual(entry.keyword, "MRImageStorage")
        self.assertEqual(entry.type, "SOP Class")

if __name__ == "__main__":
    unittest.main()



import multiprocessing
import os
import shutil
import subprocess
import sys
import tempfile
import time
import unittest

import odil

class TestStoreSCP(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self.directory = None

    def setUp(self):
        self.directory = tempfile.mkdtemp()

        with open(os.path.join(self.directory, "data.txt"), "w") as fd:
            fd.write(
                """
                (0008,0016) UI =RawDataStorage
                (0008,0018) UI [2.25.95090344942250266709587559073467305647]
                (0010,0010) PN [Doe^John]
                (0010,0020) LO [DJ001]
                """)

        subprocess.check_call([
            "dump2dcm", "--write-xfer-little",
            os.path.join(self.directory, "data.txt"),
            os.path.join(self.directory, "data.dcm")
        ])

    def tearDown(self):
        shutil.rmtree(self.directory)

    def test_store_scp_release(self):
        process = multiprocessing.Process(target=lambda: self.run_server(False))
        process.start()
        time.sleep(0.5)
        client_code = self.run_client()
        process.join(2)
        server_code = process.exitcode

        self.assertEqual(client_code, 0)
        self.assertEqual(server_code, 0)

    def run_client(self):
        command = [
            "storescu", "-ll", "error", "localhost", "11113",
            os.path.join(self.directory, "data.dcm")]

        return subprocess.call(command)

    def run_server(self, use_abort):
        called = [False]
        def store_callback(message):
            called[0] = True
            return 0

        association = odil.Association()
        association.set_tcp_timeout(1)
        association.receive_association("v4", 11113)

        store_scp = odil.StoreSCP(association)
        store_scp.set_callback(store_callback)

        message = association.receive_message()
        store_scp(message)

        termination_ok = False

        try:
            association.receive_message()
        except odil.AssociationReleased:
            termination_ok = True

        if called[0] and termination_ok:
            sys.exit(0)
        else:
            sys.exit(1)

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestVRFinder(unittest.TestCase):
    def test_default_finder(self):
        finder = odil.VRFinder()
        data_set = odil.DataSet()
        data_set.add(odil.registry.BitsAllocated, [8])
        vr = finder(
            odil.Tag(0x7fe0, 0x0010), data_set,
            odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(vr, odil.VR.OB)

if __name__ == "__main__":
    unittest.main()



import multiprocessing
import subprocess
import sys
import time
import unittest

if sys.platform == "darwin" and multiprocessing.get_start_method(True) is None:
    multiprocessing.set_start_method("fork")

import odil

class TestEchoSCP(unittest.TestCase):
    def test_echo_scp_release(self):
        process = multiprocessing.Process(target=lambda: self.run_server(False))
        process.start()
        time.sleep(0.5)
        client_code = self.run_client(False)
        process.join(2)
        server_code = process.exitcode
        
        self.assertEqual(client_code, 0)
        self.assertEqual(server_code, 0)
    
    def test_echo_scp_abort(self):
        process = multiprocessing.Process(target=lambda: self.run_server(True))
        process.start()
        time.sleep(0.5)
        client_code = self.run_client(True)
        process.join(2)
        server_code = process.exitcode
        
        self.assertEqual(client_code, 0)
        self.assertEqual(server_code, 0)
    
    def run_client(self, use_abort):
        command = ["echoscu", "-q"]
        if use_abort:
            command.append("--abort")
        command.extend(["localhost", "11113"])
        
        return subprocess.call(command)

    def run_server(self, use_abort):
        called = [False]
        def echo_callback(message):
            called[0] = True
            return 0

        association = odil.Association()
        association.set_tcp_timeout(1)
        association.receive_association("v4", 11113)

        echo_scp = odil.EchoSCP(association)
        echo_scp.set_callback(echo_callback)

        message = association.receive_message()
        echo_scp(message)
        
        termination_ok = False

        try:
            association.receive_message()
        except odil.AssociationReleased:
            if not use_abort:
                termination_ok = True
        except odil.AssociationAborted:
            if use_abort:
                termination_ok = True
        
        if called[0] and termination_ok:
            sys.exit(0)
        else:
            sys.exit(1)

if __name__ == "__main__":
    unittest.main()



import os
import sys
import unittest

import odil

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from peer_fixture_base import PeerFixtureBase

class TestFindSCU(PeerFixtureBase):
    def setUp(self):
        PeerFixtureBase.setUp(
            self, 
            [
                odil.AssociationParameters.PresentationContext(
                1, odil.registry.PatientRootQueryRetrieveInformationModelFind,
                [ odil.registry.ImplicitVRLittleEndian ], 
                odil.AssociationParameters.PresentationContext.Role.SCU)
            ]
        )

        self.query = odil.DataSet()
        self.query.add("PatientName", ["Doe^John"])
        self.query.add("QueryRetrieveLevel", ["PATIENT"])
        self.query.add("PatientID")

    def test_without_callback(self):
        find = odil.FindSCU(self.association)
        find.set_affected_sop_class(odil.registry.PatientRootQueryRetrieveInformationModelFind)
        data_sets = find.find(self.query)

        self.assertEqual(len(data_sets), 1)
        self.assertSequenceEqual(
            data_sets[0].as_string("PatientID"), [b"DJ001"])

    def test_with_callback(self):
        data_sets = []
        def callback(data_set):
            data_sets.append(data_set)

        find = odil.FindSCU(self.association)
        find.set_affected_sop_class(odil.registry.PatientRootQueryRetrieveInformationModelFind)
        find.find(self.query, callback)

        self.assertEqual(len(data_sets), 1)
        self.assertSequenceEqual(
            data_sets[0].as_string("PatientID"), [b"DJ001"])

if __name__ == "__main__":
    unittest.main()



import email
import json
import sys
import unittest

import odil

class TestSTOWRSResponse(unittest.TestCase):
    def setUp(self):
        data_set_1 = odil.DataSet()
        data_set_1.add("RetrieveURL", ["http://example.com/dicom/studies/7.8.9"])
        data_set_1.add("FailedSOPSequence", [odil.DataSet()], odil.VR.SQ)
        referenceSOPSequence_item1 = odil.DataSet()
        referenceSOPSequence_item1.add("ReferencedSOPClassUID", [odil.registry.MRImageStorage])
        referenceSOPSequence_item1.add("ReferencedSOPInstanceUID", ["1.2.3.6.1"])
        referenceSOPSequence_item1.add("RetrieveURL",
            ["http://example.com/dicom/studies/7.8.9/series/4.5/instances/1.2.3.6.1"])

        referenceSOPSequence_item2 = odil.DataSet()
        referenceSOPSequence_item2.add("ReferencedSOPClassUID", [odil.registry.RawDataStorage])
        referenceSOPSequence_item2.add("ReferencedSOPInstanceUID", ["1.2.3.6.2"])
        referenceSOPSequence_item2.add("RetrieveURL",
        ["http://example.com/dicom/studies/7.8.9/series/4.5/instances/1.2.3.6.2"])

        data_set_1.add("ReferencedSOPSequence", [referenceSOPSequence_item1, referenceSOPSequence_item2])

        self.data_set = data_set_1

    def test_constructor(self):
        response = odil.webservices.STOWRSResponse()
        self.assertEqual(len(response.get_store_instance_responses()), 0)

    def test_respondDICOM_XML(self):
        response = odil.webservices.STOWRSResponse()
        response.set_representation(odil.webservices.Utils.Representation.DICOM_XML)
        response.set_store_instance_responses(self.data_set)
        self.assertEqual(response.get_media_type(), "application/dicom+xml")
        self.assertEqual(response.get_representation(), odil.webservices.Utils.Representation.DICOM_XML)
        http = response.get_http_response()

        msg = self._http_message_to_email_message(http)

        for part in msg.walk():
            if part.get_content_type() == "application/dicom+xml":
                self.assertEqual(odil.as_xml(self.data_set, False), part.get_payload())

    def test_respondDICOM_JSON(self):
        response = odil.webservices.STOWRSResponse()
        response.set_representation(odil.webservices.Utils.Representation.DICOM_JSON)
        response.set_store_instance_responses(self.data_set)
        self.assertEqual(response.get_media_type(), "application/dicom+json")
        self.assertEqual(response.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)
        http = response.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertFalse(msg.is_multipart())
        self.assertTrue(msg.get_content_type(), "application/dicom+json")
        odil_json = [json.loads(odil.as_json(self.data_set))]
        http_json = json.loads(http.get_body().decode())
        self.assertSequenceEqual(http_json, odil_json)

    def test_equality(self):
        http_response = odil.webservices.HTTPResponse()
        http_response.set_header("Content-Type", "application/dicom+xml")
        http_response.set_status(200)
        http_response.set_reason("OK")
        http_response.set_body(odil.as_xml(self.data_set, False))

        response = odil.webservices.STOWRSResponse(http_response)
        response_2 = odil.webservices.STOWRSResponse()
        response_2.set_representation(odil.webservices.Utils.Representation.DICOM_XML)
        response_2.set_store_instance_responses(self.data_set)
        response_2.set_warning(False)
        response_2.set_reason("OK")
        self.assertEqual(response, response_2)

    def test_difference(self):
        http_response = odil.webservices.HTTPResponse()
        http_response.set_header("Content-Type", "application/dicom+xml")
        http_response.set_status(200)
        http_response.set_reason("OK")
        http_response.set_body(odil.as_xml(self.data_set, False))
        response = odil.webservices.STOWRSResponse(http_response)

        http_response_2 = odil.webservices.HTTPResponse()
        http_response_2.set_header("Content-Type", "application/dicom+xml")
        http_response_2.set_status(400)
        http_response_2.set_reason("Bad Request")
        http_response_2.set_body(odil.as_xml(self.data_set, False))
        response_2 = odil.webservices.STOWRSResponse(http_response_2)

        self.assertTrue(response != response_2)

    def test_reason(self):
        response = odil.webservices.STOWRSResponse()
        response.set_reason("Unauthorized")
        self.assertEqual(response.get_reason(), "Unauthorized")

    def test_invalid_reason(self):
        response = odil.webservices.STOWRSResponse()
        with self.assertRaises(odil.Exception):
            self.assertRaises(response.set_reason("Unrecognize"))

    def test_failureCode(self):
        response = odil.webservices.STOWRSResponse()
        response.set_failure_code(503)
        self.assertEqual(response.get_failure_code(), 503)

    def test_Representation(self):
        response = odil.webservices.STOWRSResponse()
        response.set_representation(odil.webservices.Utils.Representation.DICOM_JSON)
        self.assertTrue(response.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)

    def tearDown(self):
        self.data_set = None

    def _http_message_to_email_message(self, http_message):
        message_bytes = [
            name.encode()+b": "+value.encode()
            for (name, value) in http_message.get_headers().items()]
        message_bytes.append(http_message.get_body())
        message_bytes = b"\r\n".join(message_bytes)

        if sys.version_info[0] >= 3:
            email_message = email.message_from_bytes(message_bytes)
        else:
            email_message = email.message_from_string(message_bytes)
        return email_message

if __name__ == "__main__":
    unittest.main()



import os
import tempfile
import unittest

from io import BytesIO

import odil

class TestWriter(unittest.TestCase):
    def test_constructor_1(self):
        stream = odil.iostream(BytesIO())
        writer = odil.Writer(stream, odil.ByteOrdering.LittleEndian, False)
        self.assertEqual(writer.byte_ordering, odil.ByteOrdering.LittleEndian)
        self.assertFalse(writer.explicit_vr)
        self.assertEqual(
            writer.item_encoding, odil.Writer.ItemEncoding.ExplicitLength)
        self.assertFalse(writer.use_group_length)

    def test_constructor_2(self):
        stream = odil.iostream(BytesIO())
        writer = odil.Writer(stream, odil.registry.ExplicitVRBigEndian)
        self.assertEqual(writer.byte_ordering, odil.ByteOrdering.BigEndian)
        self.assertTrue(writer.explicit_vr)
        self.assertEqual(
            writer.item_encoding, odil.Writer.ItemEncoding.ExplicitLength)
        self.assertFalse(writer.use_group_length)

    def test_write_data_set(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Foo^Bar"])
        data_set.add("PatientID", ["FOO"])

        string_io = BytesIO()
        stream = odil.iostream(string_io)
        writer = odil.Writer(stream, odil.registry.ExplicitVRLittleEndian)
        writer.write_data_set(data_set)

        self.assertEqual(
            string_io.getvalue(),
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar "
            b"\x10\x00\x20\x00" b"LO" b"\x04\x00" b"FOO "
        )

    def test_write_tag(self):
        string_io = BytesIO()
        stream = odil.iostream(string_io)
        writer = odil.Writer(stream, odil.registry.ExplicitVRLittleEndian)
        writer.write_tag(odil.registry.PatientID)

        self.assertEqual(string_io.getvalue(), b"\x10\x00\x20\x00")

    def test_write_element(self):
        string_io = BytesIO()
        stream = odil.iostream(string_io)
        writer = odil.Writer(stream, odil.registry.ExplicitVRLittleEndian)
        writer.write_element(odil.Element(["Foo^Bar"], odil.VR.PN))
        self.assertEqual(string_io.getvalue(), b"PN\x08\x00Foo^Bar ")

    def test_write_file_stream(self):
        data_set = odil.DataSet()
        data_set.add("SOPClassUID", ["1.2.3.4"])
        data_set.add("SOPInstanceUID", ["1.2.3.4.5"])
        data_set.add("PatientName", ["Foo^Bar"])

        string_io = BytesIO()
        stream = odil.iostream(string_io)

        odil.Writer.write_file(
            data_set, stream, odil.DataSet(),
            odil.registry.ExplicitVRLittleEndian)

        data = (
            128*b"\0"+b"DICM"+
            b"\x02\x00\x00\x00" b"UL" b"\x04\x00" b"\x80\x00\x00\x00"
            b"\x02\x00\x01\x00" b"OB" b"\x00\x00" b"\x02\x00\x00\x00" b"\x00\x01"
            b"\x02\x00\x02\x00" b"UI" b"\x08\x00" b"1.2.3.4\x00"
            b"\x02\x00\x03\x00" b"UI" b"\x0a\x00" b"1.2.3.4.5\x00"
            b"\x02\x00\x10\x00" b"UI" b"\x14\x00" b"1.2.840.10008.1.2.1\x00"
            b"\x02\x00\x12\x00" b"UI" b"\x1e\x00" b"1.2.826.0.1.3680043.9.55600.0\x00"
            b"\x02\x00\x13\x00" b"SH" b"\x06\x00" b"Odil 0"
            b"\x08\x00\x16\x00" b"UI" b"\x08\x00" b"1.2.3.4\x00"
            b"\x08\x00\x18\x00" b"UI" b"\x0a\x00" b"1.2.3.4.5\x00"
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar ")

        self.assertEqual(string_io.getvalue(), data)
    
    def test_write_file_path(self):
        data_set = odil.DataSet()
        data_set.add("SOPClassUID", ["1.2.3.4"])
        data_set.add("SOPInstanceUID", ["1.2.3.4.5"])
        data_set.add("PatientName", ["Foo^Bar"])
        
        fd, path = tempfile.mkstemp()
        os.close(fd)
        
        string_io = BytesIO()
        stream = odil.iostream(string_io)

        odil.Writer.write_file(
            data_set, path, odil.DataSet(),
            odil.registry.ExplicitVRLittleEndian)

        data = (
            128*b"\0"+b"DICM"+
            b"\x02\x00\x00\x00" b"UL" b"\x04\x00" b"\x80\x00\x00\x00"
            b"\x02\x00\x01\x00" b"OB" b"\x00\x00" b"\x02\x00\x00\x00" b"\x00\x01"
            b"\x02\x00\x02\x00" b"UI" b"\x08\x00" b"1.2.3.4\x00"
            b"\x02\x00\x03\x00" b"UI" b"\x0a\x00" b"1.2.3.4.5\x00"
            b"\x02\x00\x10\x00" b"UI" b"\x14\x00" b"1.2.840.10008.1.2.1\x00"
            b"\x02\x00\x12\x00" b"UI" b"\x1e\x00" b"1.2.826.0.1.3680043.9.55600.0\x00"
            b"\x02\x00\x13\x00" b"SH" b"\x06\x00" b"Odil 0"
            b"\x08\x00\x16\x00" b"UI" b"\x08\x00" b"1.2.3.4\x00"
            b"\x08\x00\x18\x00" b"UI" b"\x0a\x00" b"1.2.3.4.5\x00"
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar ")
        
        contents = open(path, "rb").read()
        os.remove(path)
        
        self.assertEqual(contents, data)

if __name__ == "__main__":
    unittest.main()



import argparse
import sys

import odil

def main():
    parser = argparse.ArgumentParser(description="Convert a DICOM file")
    parser.add_argument("input")
    parser.add_argument("output")
    parser.add_argument("--transfer-syntax", "-t", help="Transfer syntax")
    parser.add_argument(
        "--item-length", "-i", choices=["Explicit", "Undefined"], 
        default="Explicit", help="Item length encoding")
    parser.add_argument(
        "--use-group-length", "-g", action="store_true", 
        help="Include group length tags")
    arguments = parser.parse_args()

    if arguments.transfer_syntax not in dir(odil.registry):
        parser.error("Unknown transfer syntax")
    uid = getattr(odil.registry, arguments.transfer_syntax)
    if uid not in odil.registry.uids_dictionary:
        parser.error("Unknown transfer syntax")
    if odil.registry.uids_dictionary[uid].type != "Transfer Syntax":
        parser.error("Not a transfer syntax")
    arguments.transfer_syntax = uid

    arguments.item_length = getattr(
        odil.Writer.ItemEncoding, "{}Length".format(arguments.item_length))

    convert(**arguments.__dict__)

def convert(input, output, transfer_syntax, item_length, use_group_length):
    with odil.open(input) as stream:
        header, data_set = odil.Reader.read_file(stream)

    to_remove = [
        "FileMetaInformationVersion",
        "MediaStorageSOPClassUID", "MediaStorageSOPInstanceUID", 
        "TransferSyntaxUID", 
        "ImplementationClassUID", "ImplementationVersionName"
    ]
    for name in to_remove:
        header.remove(getattr(odil.registry, name))

    odil.write(data_set, output, header, transfer_syntax, item_length, use_group_length)

if __name__ == "__main__":
    sys.exit(main())



import unittest

import odil

class TestRegistry(unittest.TestCase):
    def test_attribute(self):
        tag = odil.registry.PatientID
        self.assertEqual(tag.group, 0x0010)
        self.assertEqual(tag.element, 0x0020)

    def test_public_dictionary(self):
        tag = odil.registry.PatientID
        self.assertTrue(tag in odil.registry.public_dictionary)
        entry = odil.registry.public_dictionary[tag]
        self.assertEqual(entry.name, "Patient ID")
        self.assertEqual(entry.keyword, "PatientID")
        self.assertEqual(entry.vr, "LO")
        self.assertEqual(entry.vm, "1")

    def test_uids_dictionary(self):
        uid = "1.2.840.10008.1.2.1"
        self.assertTrue(uid in odil.registry.uids_dictionary)
        entry = odil.registry.uids_dictionary[uid]
        self.assertEqual(entry.name, "Explicit VR Little Endian")
        self.assertEqual(entry.keyword, "ExplicitVRLittleEndian")
        self.assertEqual(entry.type, "Transfer Syntax")

if __name__ == "__main__":
    unittest.main()



import argparse
import sys

import odil

def main():
    parser = argparse.ArgumentParser(description="Print content of DICOM file")
    parser.add_argument("file")
    parser.add_argument(
        "--header", "-H", action="store_true", help="Print header")
    parser.add_argument(
        "--decode-uids", "-D", action="store_true", help="Display UID names")
    arguments = parser.parse_args()

    with odil.open(arguments.file) as stream:
        header, data_set = odil.Reader.read_file(stream)
    if arguments.header:
        print_data_set(header, arguments.decode_uids)
        print
    print_data_set(data_set, arguments.decode_uids)

def print_data_set(data_set, decode_uids=False, padding=""):
    max_length = 0
    for tag in data_set.keys():
        if tag in odil.registry.public_dictionary:
            entry = odil.registry.public_dictionary[tag]
            max_length = max(max_length, len(entry.name))
        
    for tag, element in data_set.items():
        name = "{:04x},{:04x}".format(tag.group, tag.element)
        if tag in odil.registry.public_dictionary:
            entry = odil.registry.public_dictionary[tag]
            name = entry.name

        if element.is_data_set():
            value = ""
        elif element.is_binary():
            length = len(element.as_binary())
            value = "(binary, {} byte{})".format(length, "s" if length>1 else "")
        else:
            getter = None
            if element.is_int():
                getter = element.as_int
            elif element.is_real():
                getter = element.as_real
            elif element.is_string():
                getter = element.as_string
            value = [x for x in getter()]

            if decode_uids and element.vr == odil.VR.UI:
                value = [
                    odil.registry.uids_dictionary[uid].name
                        if uid in odil.registry.uids_dictionary else uid
                    for uid in value
                ]

        print "{}{}{} {:04x},{:04x} {} {}".format(
            padding,
            name, (max_length-len(name))*" ", 
            tag.group, tag.element, element.vr,
            value)

        if element.is_data_set():
            for item in element.as_data_set()[:-1]:
                print_data_set(item, decode_uids, padding+"  ")
                print
            print_data_set(element.as_data_set()[-1], padding+"  ")

if __name__ == "__main__":
    sys.exit(main())



import pickle
import re
import unittest

import odil

class TestElement(unittest.TestCase):
    def _test_sequences(self, odil_contents, python_contents):
        if python_contents and isinstance(python_contents[0], bytearray):
            self.assertSequenceEqual(
                [bytearray([x for x in item]) for item in odil_contents], 
                python_contents)
        else:
            self.assertSequenceEqual(list(odil_contents), list(python_contents))

    def _test_value(self, element, vr, contents, type_check, accessor):
        self.assertTrue(type_check(element))
        self.assertEqual(element.empty(), len(contents)==0)
        self.assertEqual(element.size(), len(contents))
        self.assertEqual(len(element), len(contents))
        # Polymorphic test
        self._test_sequences(element, contents)
        # Typed test
        self._test_sequences(accessor(element), contents)
        self.assertEqual(element.vr, vr)

        if not element.is_int():
            with self.assertRaises(odil.Exception):
                element.as_int()
        if not element.is_real():
            with self.assertRaises(odil.Exception):
                element.as_real()
        if not element.is_string():
            with self.assertRaises(odil.Exception):
                element.as_string()
        if not element.is_data_set():
            with self.assertRaises(odil.Exception):
                element.as_data_set()
        if not element.is_binary():
            with self.assertRaises(odil.Exception):
                element.as_binary()

    def _test_implicit_container(self, empty_content, vr, type_check, accessor):
        element = odil.Element(vr)
        self._test_value(element, vr, empty_content, type_check, accessor)

    def _test_container(self, contents, vr, type_check, accessor):
        element = odil.Element(contents, vr)
        self._test_value(element, vr, contents, type_check, accessor)
        
    def _test_modify(self, contents, vr, accessor):
        # Typed test
        element = odil.Element([contents[0]], vr)
        
        if isinstance(contents[0], bytearray):
            accessor(element).append(odil.Value.BinaryItem(contents[1]))
        else:
            accessor(element).append(contents[1])
        self._test_sequences(accessor(element), contents[:2])
        
        if isinstance(contents[0], bytearray):
            accessor(element)[0] = odil.Value.BinaryItem(contents[1])
        else:
            accessor(element)[0] = contents[1]
        self._test_sequences(accessor(element), [contents[1], contents[1]])
        
        # Polymorphic test
        element = odil.Element([contents[0]], vr)
        
        if isinstance(contents[0], bytearray):
            element.append(odil.Value.BinaryItem(contents[1]))
        else:
            element.append(contents[1])
        self._test_sequences(element, contents[:2])
        
        if isinstance(contents[0], bytearray):
            element[0] = odil.Value.BinaryItem(contents[1])
        else:
            element[0] = contents[1]
        self._test_sequences(element, [contents[1], contents[1]])

    def _test_clear(self, contents, vr, type_check):
        element = odil.Element(contents, vr)
        element.clear()
        self.assertTrue(type_check(element))
        self.assertTrue(element.empty())

    def _test_equality(self, contents_1, contents_2, vr_1, vr_2):
        element_1 = odil.Element(contents_1, vr_1)
        element_2 = odil.Element(contents_1, vr_1)
        element_3 = odil.Element(contents_1, vr_2)
        element_4 = odil.Element(contents_2, vr_1)

        self.assertTrue(element_1 == element_2)
        self.assertFalse(element_1 == element_3)
        self.assertFalse(element_1 == element_4)

        self.assertFalse(element_1 != element_2)
        self.assertTrue(element_1 != element_3)
        self.assertTrue(element_1 != element_4)
    
    def _test_pickle(self, contents, vr, accessor):
        element = odil.Element(contents, vr)
        self.assertSequenceEqual(
            accessor(pickle.loads(pickle.dumps(element))), accessor(element))
        self.assertEqual(pickle.loads(pickle.dumps(element)).vr, vr)
    
    def _test(
            self, 
            empty_content, contents, other_contents, vr, other_vr, 
            type_check, accessor):
        self._test_implicit_container(empty_content, vr, type_check, accessor)
        self._test_container(contents, vr, type_check, accessor)
        self._test_modify(contents, vr, accessor)
        self._test_clear(contents, vr, type_check)
        self._test_equality(contents, other_contents, vr, other_vr)
        self._test_pickle(contents, vr, accessor)
        
    def test_implicit_type(self):
        for vr in dir(odil.VR):
            if re.match(r"^[A-Z]{2}$", vr) is None:
                continue

            vr = getattr(odil.VR, vr)
            if odil.is_int(vr):
                self._test_implicit_container(
                    odil.Value.Integers(), vr, 
                    odil.Element.is_int, odil.Element.as_int)
            elif odil.is_real(vr):
                self._test_implicit_container(
                    odil.Value.Reals(), vr, 
                    odil.Element.is_real, odil.Element.as_real)
            elif odil.is_string(vr):
                self._test_implicit_container(
                    odil.Value.Strings(), vr, 
                    odil.Element.is_string, odil.Element.as_string)
            elif vr == odil.VR.SQ:
                self._test_implicit_container(
                    odil.Value.DataSets(), vr, 
                    odil.Element.is_data_set, odil.Element.as_data_set)
            elif odil.is_binary(vr):
                self._test_implicit_container(
                    odil.Value.Binary(), vr, 
                    odil.Element.is_binary, odil.Element.as_binary)

    def test_integers(self):
        self._test(
            odil.Value.Integers(), [1234, 5678, 9012], [3456, 7890],
            odil.VR.US, odil.VR.UL,
            odil.Element.is_int, odil.Element.as_int)

    def test_reals(self):
        self._test(
            odil.Value.Reals(), [12.34, 56.78, 90.12], [1., 2.],
            odil.VR.FD, odil.VR.DS,
            odil.Element.is_real, odil.Element.as_real)

    def test_strings(self):
        self._test(
            odil.Value.Strings(), [b"foo", b"bar", b"baz"], [b"plip", b"plop"],
            odil.VR.CS, odil.VR.UT,
            odil.Element.is_string, odil.Element.as_string)
        
    def test_data_sets(self):
        data_set_1 = odil.DataSet(PatientID=["DJ1234"])
        data_set_2 = odil.DataSet(EchoTime=[100])
        data_set_3 = odil.DataSet(PixelSpacing=[2.3, 4.5])

        self._test(
            odil.Value.DataSets(), 
            [data_set_1, data_set_2, data_set_3], [data_set_2, data_set_1],
            odil.VR.SQ, odil.VR.UN,
            odil.Element.is_data_set, odil.Element.as_data_set)

    def test_binary(self):
        self._test(
            odil.Value.Binary(), 
            [bytearray([0x01, 0x02]), bytearray([0x03]), bytearray([0x04, 0x05])],
            [bytearray([0x04]), bytearray([0x05, 0x06])],
            odil.VR.OB, odil.VR.OW,
            odil.Element.is_binary, odil.Element.as_binary)
    
    def test_pickle_empty(self):
        self.assertEqual(
            pickle.loads(pickle.dumps(odil.Element(odil.VR.CS))),
            odil.Element(odil.VR.CS))

if __name__ == "__main__":
    unittest.main()



from __future__ import print_function
import argparse
import logging

import odil

from .print_ import find_max_name_length, print_data_set

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "find", help="DICOM query (C-FIND)",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("host", help="Remote host address")
    parser.add_argument("port", type=int, help="Remote host port")
    parser.add_argument(
        "calling_ae_title", help="AE title of the calling application")
    parser.add_argument(
        "called_ae_title", help="AE title of the called application")
    parser.add_argument(
        "level", choices=["patient", "study"], help="Root object of the query")
    parser.add_argument("keys", nargs="+", help="Query keys")
    parser.add_argument(
        "--decode-uids", "-u", action="store_true",
        help="Print human-friendly name of known UIDs")
    parser.set_defaults(function=find)
    return parser

def find(host, port, calling_ae_title, called_ae_title, level, keys, decode_uids):
    query = odil.DataSet()
    for key in keys:
        if "=" in key:
            key, value = key.split("=", 1)
            value = value.split("\\")
        else:
            value = None
        
        tag = getattr(odil.registry, key)
        
        if value is not None:
            vr = odil.registry.public_dictionary[tag].vr
            if vr in ["DS", "FL", "FD"]:
                value = [float(x) for x in value]
            elif vr in ["IS", "SL", "SS", "UL", "US"]:
                value = [int(x) for x in value]
            
            query.add(tag, value)
        else:
            query.add(tag)
    
    sop_class = getattr(
        odil.registry,
        "{}RootQueryRetrieveInformationModelFind".format(level.capitalize()))
    
    find_pc = odil.AssociationParameters.PresentationContext(
        1, sop_class,
        [
            odil.registry.ImplicitVRLittleEndian,
            odil.registry.ExplicitVRLittleEndian
        ], odil.AssociationParameters.PresentationContext.Role.SCU
    )
    
    association = odil.Association()
    association.set_peer_host(host)
    association.set_peer_port(port)
    association.update_parameters()\
        .set_calling_ae_title(calling_ae_title)\
        .set_called_ae_title(called_ae_title) \
        .set_presentation_contexts([find_pc])
    association.associate()
    logging.info("Association established")
    
    find = odil.FindSCU(association)
    find.set_affected_sop_class(sop_class)
    data_sets = find.find(query)
    print("{} answer{}".format(len(data_sets), "s" if len(data_sets)>1 else ""))

    max_length = 0
    for data_set in data_sets:
        max_length = max(max_length, find_max_name_length(data_set))

    for data_set in data_sets:
        print_data_set(data_set, decode_uids, "", max_length, odil.Value.Strings())
        print()

    association.release()
    logging.info("Association released")



import os
import sys
import unittest

import odil

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from peer_fixture_base import PeerFixtureBase

class TestEchoSCU(PeerFixtureBase):
    def setUp(self):
        PeerFixtureBase.setUp(
            self, 
            [
                odil.AssociationParameters.PresentationContext(
                    3, odil.registry.Verification,
                    [ odil.registry.ImplicitVRLittleEndian ], 
                    odil.AssociationParameters.PresentationContext.Role.SCU)
            ]
        )
    
    def test_echo(self):
        echo = odil.EchoSCU(self.association)
        echo.echo()

if __name__ == "__main__":
    unittest.main()



import pickle
import unittest

import odil

class TestDataSet(unittest.TestCase):
    def test_empty_constructor(self):
        data_set = odil.DataSet()
        self.assertTrue(data_set.empty())
        self.assertEqual(data_set.size(), 0)
        self.assertEqual(len(data_set), 0)
        self.assertEqual(len(data_set.get_transfer_syntax()), 0)

    def test_transfer_syntax_constructor(self):
        data_set = odil.DataSet(odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(
            data_set.get_transfer_syntax().encode(),
            odil.registry.ExplicitVRLittleEndian)
    
    def test_kwargs_constructor(self):
        data_set = odil.DataSet(
            odil.registry.ImplicitVRLittleEndian,
            PatientName=["Doe^John"], PatientWeight=[70],
            ReferencedStudySequence=[
                odil.DataSet(StudyInstanceUID=["1.2.3"])],
            PixelData=[odil.Value.BinaryItem([1,2,3])])
        
        self.assertEqual(data_set.size(), 4)
        
        self._test_element_value(
            data_set, "PatientName", [b"Doe^John"], odil.VR.PN,
            odil.DataSet.is_string, odil.DataSet.as_string)
        self._test_element_value(
            data_set, "PatientWeight", [70], odil.VR.DS,
            odil.DataSet.is_int, odil.DataSet.as_int)
        self._test_element_value(
            data_set, "ReferencedStudySequence", [
               odil.DataSet(StudyInstanceUID=["1.2.3"])
            ], odil.VR.SQ,
            odil.DataSet.is_data_set, odil.DataSet.as_data_set)
        self._test_element_value(
            data_set, "PixelData", [odil.Value.BinaryItem([1,2,3])], odil.VR.OW,
            odil.DataSet.is_binary, odil.DataSet.as_binary)
        
        with self.assertRaises(odil.Exception):
            odil.DataSet(FooBar=["foo"])
    
    def test_transfer_syntax(self):
        data_set = odil.DataSet()
        data_set.set_transfer_syntax(odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(
            data_set.get_transfer_syntax().encode(),
            odil.registry.ExplicitVRLittleEndian)

    def test_empty_element_tag(self):
        tag = odil.registry.PatientName
        data_set = odil.DataSet()
        data_set.add(tag)

        self.assertFalse(data_set.empty())
        self.assertEqual(data_set.size(), 1)
        self.assertEqual(len(data_set), 1)

        self.assertEqual(data_set.get_vr(tag), odil.VR.PN)
        self.assertTrue(data_set.empty(tag))
        self.assertEqual(data_set.size(tag), 0)

    def _test_sequences(self, odil_contents, python_contents):
        if python_contents and isinstance(python_contents[0], bytearray):
            self.assertSequenceEqual(
                [bytearray([x for x in item]) for item in odil_contents], 
                python_contents)
        else:
            self.assertSequenceEqual(list(odil_contents), list(python_contents))

    def _test_element_value(
            self, data_set, tag, contents, vr, type_check, accessor):
        self.assertTrue(data_set.has(tag))
        self.assertEqual(data_set.empty(tag), len(contents)==0)
        self.assertEqual(data_set.size(tag), len(contents))
        self.assertTrue(type_check(data_set, tag))
        self.assertEqual(data_set[tag].vr, vr)
        self._test_sequences(accessor(data_set, tag), contents)

        if not data_set.is_int(tag):
            with self.assertRaises(odil.Exception):
                data_set.as_int(tag)
        if not data_set.is_real(tag):
            with self.assertRaises(odil.Exception):
                data_set.as_real(tag)
        if not data_set.is_string(tag):
            with self.assertRaises(odil.Exception):
                data_set.as_string(tag)
        if not data_set.is_data_set(tag):
            with self.assertRaises(odil.Exception):
                data_set.as_data_set(tag)
        if not data_set.is_binary(tag):
            with self.assertRaises(odil.Exception):
                data_set.as_binary(tag)

    def _test_implicit_container(
            self, tag, empty_content, vr, type_check, accessor):

        # Implicit empty content
        data_set = odil.DataSet()
        data_set.add(tag)
        self._test_element_value(
            data_set, tag, empty_content, vr, type_check, accessor)

        # Explicit empty content
        data_set = odil.DataSet()
        data_set.add(tag, empty_content)
        self._test_element_value(
            data_set, tag, empty_content, vr, type_check, accessor)

    def _test_container_no_vr(self, tag, contents, vr, type_check, accessor):
        data_set = odil.DataSet()
        data_set.add(tag, contents)
        self._test_element_value(
            data_set, tag, contents, vr, type_check, accessor)

    def _test_container_vr(self, tag, contents, vr, type_check, accessor):
        data_set = odil.DataSet()
        data_set.add(tag, contents, vr)
        self._test_element_value(
            data_set, tag, contents, vr, type_check, accessor)

    def _test_modify(self, tag, contents, accessor):
        # Typed test
        data_set = odil.DataSet()
        data_set.add(tag, [contents[0]])
        if isinstance(contents[0], bytearray):
            accessor(data_set, tag).append(
                odil.Value.BinaryItem(contents[1]))
        else:
            accessor(data_set, tag).append(contents[1])
        self._test_sequences(accessor(data_set, tag), contents)
        
        # Polymorphic test
        data_set = odil.DataSet()
        data_set.add(tag, [contents[0]])
        if isinstance(contents[0], bytearray):
            data_set[tag].append(odil.Value.BinaryItem(contents[1]))
        else:
            data_set[tag].append(contents[1])
        self._test_sequences(data_set[tag], contents)

    def _test_clear(self, tag, contents, type_check):
        data_set = odil.DataSet()
        data_set.add(tag, contents)
        data_set.clear(tag)
        self.assertTrue(type_check(data_set, tag))
        self.assertTrue(data_set.empty(tag))

    def _test_item(self, tag, contents, type_check, accessor):
        data_set = odil.DataSet()
        data_set.add(tag, contents)

        self._test_sequences(accessor(data_set[tag]), contents)

        data_set[tag] = odil.Element(contents[:1], data_set.get_vr(tag))
        self._test_sequences(accessor(data_set[tag]), contents[:1])

    def _test_element(
            self, tag, empty_content, contents, vr,
            data_set_type_check, data_set_accessor,
            element_type_check, element_accessor):
        self._test_implicit_container(
            tag, empty_content, vr, data_set_type_check, data_set_accessor)

        self._test_container_no_vr(
            tag, contents, vr, data_set_type_check, data_set_accessor)
        self._test_container_vr(
            tag, contents, vr, data_set_type_check, data_set_accessor)

        self._test_modify(tag, contents, data_set_accessor)

        self._test_clear(tag, contents, data_set_type_check)

        self._test_item(tag, contents, element_type_check, element_accessor)

    def test_int(self):
        self._test_element(
            odil.registry.Rows, odil.Value.Integers(), [1234, 5678], odil.VR.US,
            odil.DataSet.is_int, odil.DataSet.as_int,
            odil.Element.is_int, odil.Element.as_int)

    def test_real(self):
        self._test_element(
            odil.registry.B1rms, odil.Value.Reals(), 
            [12.34, 56.78], odil.VR.FL,
            odil.DataSet.is_real, odil.DataSet.as_real,
            odil.Element.is_real, odil.Element.as_real)
    
    def test_real_string(self):
        self._test_element(
            odil.registry.FieldOfViewOrigin, odil.Value.Reals(), 
            [12.34, 56.78], odil.VR.DS,
            odil.DataSet.is_real, odil.DataSet.as_real,
            odil.Element.is_real, odil.Element.as_real)

    def test_string(self):
        self._test_element(
            odil.registry.PatientID, odil.Value.Strings(), [b"foo", b"bar"],
            odil.VR.LO,
            odil.DataSet.is_string, odil.DataSet.as_string,
            odil.Element.is_string, odil.Element.as_string)

    def test_data_set(self):
        data_set_1 = odil.DataSet()
        data_set_1.add("PatientID", ["DJ1234"])
    
        data_set_2 = odil.DataSet()
        data_set_2.add("EchoTime", [100])

        self._test_element(
            odil.registry.ReferencedStudySequence, odil.Value.DataSets(), 
            [data_set_1, data_set_2], odil.VR.SQ,
            odil.DataSet.is_data_set, odil.DataSet.as_data_set,
            odil.Element.is_data_set, odil.Element.as_data_set)

    def test_binary(self):
        self._test_element(
            odil.registry.BadPixelImage, odil.Value.Binary(), 
            [bytearray([0x01, 0x02]), bytearray([0x03])], odil.VR.OB,
            odil.DataSet.is_binary, odil.DataSet.as_binary,
            odil.Element.is_binary, odil.Element.as_binary)

    def test_getitem(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        self.assertSequenceEqual(
            data_set["PatientName"].as_string(), [b"Doe^John"])
        self.assertRaises(Exception, lambda x: data_set["PatientID"])
    
    def test_delitem(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        del data_set[odil.registry.PatientName]
        self.assertTrue(data_set.empty())
        
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        del data_set["PatientName"]
        self.assertTrue(data_set.empty())
    
    def test_contains(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        self.assertTrue("PatientName" in data_set)
        self.assertTrue(odil.registry.PatientName in data_set)
        self.assertTrue("PatientID" not in data_set)
        self.assertTrue(odil.registry.PatientID not in data_set)
    
    def test_get(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        
        self.assertSequenceEqual(
            data_set.get(odil.registry.PatientName).as_string(), [b"Doe^John"])
        self.assertSequenceEqual(
            data_set.get("PatientName").as_string(), [b"Doe^John"])
        
        self.assertEqual(data_set.get(odil.registry.PatientID), None)
        self.assertEqual(data_set.get("PatientID"), None)
        
        self.assertEqual(data_set.get(odil.registry.PatientID, 123), 123)
        self.assertEqual(data_set.get("PatientID", 123), 123)
        
    def test_iter(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientID", ["DJ123"])
        self.assertEqual(
            [x.get_name() for x in data_set],
            ["PatientName", "PatientID"])

    def test_keys(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientID", ["DJ123"])
        self.assertEqual(
            [x.get_name() for x in data_set.keys()],
            ["PatientName", "PatientID"])
        self.assertEqual(
            [x.get_name() for x in list(data_set)],
            ["PatientName", "PatientID"])
        self.assertEqual(
            [x.get_name() for x in data_set],
            ["PatientName", "PatientID"])

    def test_values(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientID", ["DJ123"])
        self.assertEqual(
            [
                [item for item in element.as_string()]
                for element in data_set.values()],
            [[b"Doe^John"], [b"DJ123"]])

    def test_items(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientID", ["DJ123"])
        self.assertSequenceEqual(
            [
                [tag.get_name(), [item for item in element.as_string()]]
                for tag, element in data_set.items()],
            [["PatientName", [b"Doe^John"]], ["PatientID", [b"DJ123"]]])
    
    def test_update(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientSex", ["M"])
        
        other = odil.DataSet()
        data_set.add("PatientName", ["Bloggs^Joe"])
        other.add("PatientID", ["BJ123"])
        other.add("IssuerOfPatientID", ["Hospital"])
        
        data_set.update(other)
        
        self.assertSequenceEqual(
            [
                [tag.get_name(), [item for item in element.as_string()]]
                for tag, element in data_set.items()],
            [
                ["PatientName", [b"Bloggs^Joe"]], ["PatientID", [b"BJ123"]],
                ["IssuerOfPatientID", [b"Hospital"]], ["PatientSex", [b"M"]]])
    
    def test_pickle(self):
        data_set = odil.DataSet()
        data_set.add("PatientName", ["Doe^John"])
        data_set.add("PatientID", ["DJ123"])
        self.assertEqual(pickle.loads(pickle.dumps(data_set)), data_set)

if __name__ == "__main__":
    unittest.main()



import email
import json
import sys
import unittest

import odil

class TestQIDORSResponse(unittest.TestCase):
    data_set_1 = odil.DataSet()
    data_set_1.add("SOPClassUID", [odil.registry.RawDataStorage])
    data_set_1.add("SOPInstanceUID", ["1.2.3.4"])
    data_set_1.add("PatientID", ["DJ1234"])
    data_set_1.add("PixelSpacing", [1.5, 2.5])

    data_set_2 = odil.DataSet()
    data_set_2.add("SOPClassUID", [odil.registry.MRImageStorage])
    data_set_2.add("SOPInstanceUID", ["1.2.3.5"])
    data_set_2.add("PatientName", ["Doe^John"])
    data_set_2.add("PatientAge", ["042Y"])

    data_sets =  [data_set_1, data_set_2]

    def test_constructor(self):
        response = odil.webservices.QIDORSResponse()
        self.assertEqual(len(response.get_data_sets()), 0)

    def test_data_sets(self):
        response = odil.webservices.QIDORSResponse()
        response.set_data_sets(self.data_sets)
        self.assertSequenceEqual(response.get_data_sets(), self.data_sets)

    def test_Response_DicomJson(self):
        response = odil.webservices.QIDORSResponse()
        response.set_representation(odil.webservices.Utils.Representation.DICOM_JSON)
        response.set_data_sets(self.data_sets)
        self.assertEqual(response.get_media_type(), "application/dicom+json")
        self.assertEqual(response.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)
        http = response.get_http_response()
        content_type = http.get_header("Content-Type")
        self.assertEqual(content_type, "application/dicom+json")
        json_http = json.loads(http.get_body().decode())
        json_ds = [ json.loads(odil.as_json(x)) for x in self.data_sets ]
        self.assertEqual(json_ds, json_http)

    def test_Response_DicomXML(self):
        response = odil.webservices.QIDORSResponse()
        response.set_representation(odil.webservices.Utils.Representation.DICOM_XML)
        response.set_data_sets(self.data_sets)
        self.assertEqual(response.get_media_type(), "application/dicom+xml")
        self.assertEqual(response.get_representation(), odil.webservices.Utils.Representation.DICOM_XML)
        http = response.get_http_response()

        message_bytes = [
            name.encode()+b": "+value.encode()
            for (name, value) in http.get_headers().items()]
        message_bytes.append(http.get_body())
        message_bytes = b"\r\n".join(message_bytes)
        if sys.version_info[0] >= 3:
            msg = email.message_from_bytes(message_bytes)
        else:
            msg = email.message_from_string(message_bytes)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_content_type() == "application/dicom+xml":
                self.assertEqual(odil.as_xml(self.data_sets[i], False), part.get_payload())
                i = i+1
        self.assertEqual(i, len(self.data_sets))

    def test_empty_response(self):
        response = odil.webservices.QIDORSResponse()
        http = response.get_http_response()
        self.assertEqual(http.get_status(), 204)
        self.assertEqual(http.get_reason(), "No Content")


if __name__ == "__main__":
    unittest.main()



#encoding: utf-8
import pickle
import unittest

import odil

class TestValue(unittest.TestCase):
    def _test_sequences(self, odil_contents, python_contents):
        self.assertEqual(len(odil_contents), len(python_contents))

        if python_contents and isinstance(python_contents[0], bytearray):
            self.assertSequenceEqual(
                [bytearray([x for x in item]) for item in odil_contents], 
                python_contents)

            # Access from start
            self.assertEqual(bytearray(odil_contents[0]), python_contents[0])

            # Access from end
            self.assertEqual(bytearray(odil_contents[-1]), python_contents[-1])

            # Slice
            self.assertSequenceEqual(
                [bytearray([x for x in item]) for item in odil_contents[0:-1]],
                python_contents[0:-1])
        else:
            self.assertSequenceEqual(list(odil_contents), list(python_contents))

            if python_contents:
                # Access from start
                self.assertEqual(odil_contents[0], python_contents[0])

                # Access from end
                self.assertEqual(odil_contents[-1], python_contents[-1])

                # Slice
                self.assertSequenceEqual(
                    odil_contents[2:0:-1], python_contents[2:0:-1])
        
    def _test_contents(self, value, contents, type_, accessor):
        self.assertEqual(value.type, type_)
        self.assertEqual(value.empty(), len(contents) == 0)
        self.assertEqual(value.size(), len(contents))
        self.assertEqual(len(value), len(contents))
        # Polymorphic test
        self._test_sequences(value, contents)
        # Typed test
        self._test_sequences(accessor(value), contents)

        if type_ != odil.Value.Type.Integers:
            with self.assertRaises(odil.Exception):
                value.as_integers()
        if type_ != odil.Value.Type.Reals:
            with self.assertRaises(odil.Exception):
                value.as_reals()
        if type_ != odil.Value.Type.Strings:
            with self.assertRaises(odil.Exception):
                value.as_strings()
        if type_ != odil.Value.Type.DataSets:
            with self.assertRaises(odil.Exception):
                value.as_data_sets()
        if type_ != odil.Value.Type.Binary:
            with self.assertRaises(odil.Exception):
                value.as_binary()

    def _test_container(self, contents, type_, accessor):
        value = odil.Value(contents)
        self._test_contents(value, contents, type_, accessor)

    def _test_modify(self, contents, accessor):
        # Typed test
        value = odil.Value([contents[0]])
        
        if isinstance(contents[0], bytearray):
            accessor(value).append(odil.Value.BinaryItem(contents[1]))
        else:
            accessor(value).append(contents[1])
        self._test_sequences(accessor(value), contents[:2])
        
        if isinstance(contents[0], bytearray):
            accessor(value)[0] = odil.Value.BinaryItem(contents[1])
        else:
            accessor(value)[0] = contents[1]
        self._test_sequences(accessor(value), [contents[1], contents[1]])
        
        # Polymorphic test
        value = odil.Value([contents[0]])
        
        if isinstance(contents[0], bytearray):
            value.append(odil.Value.BinaryItem(contents[1]))
        else:
            value.append(contents[1])
        self._test_sequences(value, contents[:2])
        
        if isinstance(contents[0], bytearray):
            value[0] = odil.Value.BinaryItem(contents[1])
        else:
            value[0] = contents[1]
        self._test_sequences(value, [contents[1], contents[1]])

    def _test_clear(self, contents, type_):
        value = odil.Value(contents)
        value.clear()
        self.assertEqual(value.type, type_)
        self.assertTrue(value.empty())

    def _test_equality(self, contents_1, contents_2):
        value_1 = odil.Value(contents_1)
        value_2 = odil.Value(contents_1)
        value_3 = odil.Value(contents_2)
        value_4 = odil.Value(contents_2)

        self.assertTrue(value_1 == value_2)
        self.assertFalse(value_1 == value_3)
        self.assertFalse(value_1 == value_4)

        self.assertFalse(value_1 != value_2)
        self.assertTrue(value_1 != value_3)
        self.assertTrue(value_1 != value_4)
    
    def _test_pickle(self, contents, accessor):
        value = odil.Value(contents)
        self.assertSequenceEqual(
            accessor(pickle.loads(pickle.dumps(value))), accessor(value))
    
    def _test(self, empty_content, contents, other_contents, type_, accessor):
        self._test_container(empty_content, type_, accessor)
        self._test_container(contents, type_, accessor)
        self._test_modify(contents, accessor)
        self._test_clear(contents, type_)
        self._test_equality(contents, other_contents)
        self._test_pickle(contents, accessor)

    def test_integers(self):
        self._test(
            odil.Value.Integers(), [1234, 5678, 9012], [9012, 3456], 
            odil.Value.Type.Integers, odil.Value.as_integers)
        
    def test_reals(self):
        self._test(
            odil.Value.Reals(), [12.34, 56.78, 90.12], [1., 2.],
            odil.Value.Type.Reals, odil.Value.as_reals)

    def test_strings(self):
        self._test(
            odil.Value.Strings(), 
            [b"foo", b"bar", b"baz"], [b"plip", b"plop"],
            odil.Value.Type.Strings, odil.Value.as_strings)

    def test_data_sets(self):
        data_set_1 = odil.DataSet(PatientID=["DJ1234"])
        data_set_2 = odil.DataSet(EchoTime=[100])
        data_set_3 = odil.DataSet(PixelSpacing=[2.3, 4.5])

        self._test(
            odil.Value.DataSets(), 
            [data_set_1, data_set_2, data_set_3], [data_set_2, data_set_1],
            odil.Value.Type.DataSets, odil.Value.as_data_sets)

    def test_binary(self):
        self._test(
            odil.Value.Binary(), 
            [bytearray([0x01, 0x02]), bytearray([0x03]), bytearray([0x04, 0x05])],
            [bytearray([0x04]), bytearray([0x05, 0x06])],
            odil.Value.Type.Binary, odil.Value.as_binary)
        
    def test_unknown_constructor(self):
        class Foo(object): pass
        items = [Foo()]
        with self.assertRaises(odil.Exception):
            odil.Value(items)

class TestValueIntegers(unittest.TestCase):
    def test_empty_constructor(self):
        data = odil.Value.Integers()
        self.assertEqual([x for x in data], [])

    def test_sequence_constructor(self):
        items = [1, 2, 3]
        data = odil.Value.Integers(items)
        self.assertEqual([x for x in data], items)

class TestValueReals(unittest.TestCase):
    def test_empty_constructor(self):
        data = odil.Value.Reals()
        self.assertEqual([x for x in data], [])

    def test_sequence_constructor(self):
        items = [1.1, 2, 3.3]
        data = odil.Value.Reals(items)
        self.assertEqual([x for x in data], items)

class TestValueStrings(unittest.TestCase):
    def test_empty_constructor(self):
        data = odil.Value.Strings()
        self.assertEqual([x for x in data], [])

    def test_sequence_constructor(self):
        items = [b"foo", b"bar"]
        data = odil.Value.Strings(items)
        self.assertEqual([x for x in data], items)

class TestValueDataSets(unittest.TestCase):
    def test_empty_constructor(self):
        data = odil.Value.DataSets()
        self.assertEqual([x for x in data], [])

    def test_sequence_constructor(self):
        items = [odil.DataSet(), odil.DataSet()]
        data = odil.Value.DataSets(items)
        self.assertEqual([x for x in data], items)

class TestValueBinary(unittest.TestCase):
    def test_empty_constructor(self):
        data = odil.Value.Binary()
        self.assertEqual([x for x in data], [])

    def test_sequence_constructor(self):
        items = [odil.Value.BinaryItem([1, 2, 3])]
        data = odil.Value.Binary(items)
        self.assertEqual([x for x in data[0]], [x for x in items[0]])

    def test_buffer(self):
        item = odil.Value.BinaryItem([1, 2, 3])
        memory_view = item.get_memory_view()
        self.assertTrue(isinstance(memory_view, memoryview))


if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestQIDORSRequest(unittest.TestCase):
    base_url_http = odil.webservices.URL("http", "example.com", "/dicom", "", "")
    full_url = odil.webservices.URL(
        "http",
        "example.com",
        "/dicom/studies/1.2/instances",
        "PatientName=TOTO&"
        "SharedFunctionalGroupsSequence.EffectiveEchoTime=10.5&"
        "includefield=00200020&includefield=52009229.00200035",
        ""
    )

    def test_url_constructor(self):
        request = odil.webservices.QIDORSRequest(self.base_url_http)
        self.assertEqual(request.get_base_url(), self.base_url_http)
        self.assertEqual(request.get_media_type(), "")
        self.assertEqual(request.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)
        self.assertEqual(request.get_url(), odil.webservices.URL())
        self.assertEqual(request.get_selector(), odil.webservices.Selector())
        self.assertEqual(request.get_query_data_set(), None)
        self.assertFalse(request.get_fuzzymatching())
        self.assertEqual(request.get_limit(), -1)
        self.assertEqual(request.get_offset(), 0)

    def test_base_url(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        self.assertEqual(request.get_base_url(), self.base_url_http)

    def test_set_base_url(self):
        request = odil.webservices.QIDORSRequest(odil.webservices.URL())

    def test_full_url(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        full_url_alphabetic = odil.webservices.URL(
            "http",
            "example.com",
            "/dicom/studies/1.2/instances",
            "PatientName=TOTO&"
            "SharedFunctionalGroupsSequence.EffectiveEchoTime=10.5&"
            "includefield=PatientOrientation&"
            "includefield=SharedFunctionalGroupsSequence.ImageOrientation&"
            "fuzzymatching=false",
            ""
        )
        # Check for whole url
        self.assertEqual(request.get_url(), full_url_alphabetic)

    def test_selector(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        # Check For Selector
        self.assertEqual(request.get_selector().get_path(False), "/studies/1.2/instances")

    def test_query_dataset(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        # Check For DataSet
        dataset = odil.DataSet()
        dataset.add(odil.Tag("PatientName"),["TOTO"])
        dataset.add(odil.Tag("00200020"))
        shared = odil.DataSet()
        shared.add(odil.Tag("EffectiveEchoTime"), [10.5])
        shared.add(odil.Tag("00200035"))
        dataset.add(odil.Tag("SharedFunctionalGroupsSequence"), [shared])
        self.assertEqual(request.get_query_data_set(), dataset)

    def test_fuzzy_limit_offset(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        # Check for fuzzymatching
        self.assertFalse(request.get_fuzzymatching())
        # Check for limit - offset
        self.assertEqual(request.get_limit(), -1)
        self.assertEqual(request.get_offset(), 0)

    def test_media_type_json(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request)
        self.assertEqual(request.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)

    def test_media_type_xml(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "multipart/related;type=application/dicom+xml")
        request = odil.webservices.QIDORSRequest(http_request)
        self.assertEqual(request.get_representation(), odil.webservices.Utils.Representation.DICOM_XML)

    def test_media_type_unrecognized(self):
        http_request = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request.set_header("Accept", "multipart/related;type=application/dicom+json")
        with self.assertRaises(odil.Exception):
            odil.webservices.QIDORSRequest(http_request)

    def test_request_data_set(self):
        # Selector
        selector = odil.webservices.Selector()
        selector.set_study("1.2").set_instance("")
        # DataSet
        dataset = odil.DataSet()
        dataset.add(odil.Tag("PatientName"),["TOTO"])
        dataset.add(odil.Tag("00200020"))
        shared = odil.DataSet()
        shared.add(odil.Tag("StudyDate"), [20130509])
        shared.add(odil.Tag("00200035"))
        dataset.add(odil.Tag("SharedFunctionalGroupsSequence"), [shared])

        request = odil.webservices.QIDORSRequest(self.base_url_http)
        request.request_datasets(
            odil.webservices.Utils.Representation.DICOM_XML,
            selector,
            dataset
        )

        full_url_alphabetic_tags = odil.webservices.URL(
            "http",
            "example.com",
            "/dicom/studies/1.2/instances",
            "PatientName=TOTO&"
            "SharedFunctionalGroupsSequence.StudyDate=20130509&"
            "includefield=PatientOrientation&"
            "includefield=SharedFunctionalGroupsSequence.ImageOrientation&"
            "fuzzymatching=false",
            ""
        )
        self.assertEqual(request.get_http_request().get_target(), full_url_alphabetic_tags)
        self.assertEqual(request.get_http_request().get_method(), "GET")

    def test_equality(self):
        full_url_cpy = odil.webservices.URL(
            "http",
            "example.com",
            "/dicom/studies/1.2/instances",
            "SharedFunctionalGroupsSequence.EffectiveEchoTime=10.5&"
            "includefield=00200020&"
            "includefield=52009229.00200035&"
            "PatientName=TOTO"
            ""
        )
        http_request_base = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request_base.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request_base)
        http_request_cpy = odil.webservices.HTTPRequest("GET", full_url_cpy)
        http_request_cpy.set_header("Accept", "application/dicom+json")
        request_cpy = odil.webservices.QIDORSRequest(http_request_cpy)

        self.assertTrue(request == request_cpy)

    def test_difference(self):
        full_url_cpy = odil.webservices.URL(
            "http",
            "example.com",
            "/dicom/studies/1.2/instances",
            "SharedFunctionalGroupsSequence.EffectiveEchoTime=10.5&"
            "includefield=00200020&"
            "includefield=52009229.00200035&"
            "PatientName=TUTU"
            ""
        )
        http_request_base = odil.webservices.HTTPRequest("GET", self.full_url)
        http_request_base.set_header("Accept", "application/dicom+json")
        request = odil.webservices.QIDORSRequest(http_request_base)
        http_request_cpy = odil.webservices.HTTPRequest("GET", full_url_cpy)
        http_request_cpy.set_header("Accept", "application/dicom+json")
        request_cpy = odil.webservices.QIDORSRequest(http_request_cpy)

        self.assertIsNot(request, request_cpy)

if __name__ == "__main__":
    unittest.main()



from __future__ import print_function
import argparse
import logging

import odil

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "print", help="Print the contents of data sets",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("inputs", nargs="+", metavar="FILE", help="Input files")
    parser.add_argument(
        "--print-header", "-H", action="store_true",
        help="Print the header as well as the data set")
    parser.add_argument(
        "--decode-uids", "-u", action="store_true",
        help="Print human-friendly name of known UIDs")
    parser.set_defaults(function=print_)
    return parser

def print_(inputs, print_header, decode_uids):
    for input in inputs:
        logging.info("Printing {}".format(input))
        with odil.open(input) as stream:
            header, data_set = odil.Reader.read_file(stream)

        max_length = find_max_name_length(data_set)
        if print_header:
            max_length = max(max_length, find_max_name_length(header))

        if print_header:
            print_data_set(
                header, decode_uids, "", max_length, odil.Value.Strings())
            print()
        print_data_set(
            data_set, decode_uids, "", max_length, odil.Value.Strings())

def print_data_set(
        data_set, decode_uids, padding, max_length, specific_character_set):
    for tag, element in data_set.items():
        name = "{:04x},{:04x}".format(tag.group, tag.element)
        if tag in odil.registry.public_dictionary:
            entry = odil.registry.public_dictionary[tag]
            name = entry.name

        if tag == odil.registry.SpecificCharacterSet:
            specific_character_set = element.as_string()

        if element.is_data_set():
            value = "(sequence, {} item{})".format(
                len(element), "s" if len(element)>1 else "")
        elif element.is_binary():
            lengths = [len(x) for x in element.as_binary()]
            value = "(binary, {} item{}, {} byte{})".format(
                len(element), "s" if len(element)>1 else "",
                "+".join(str(x) for x in lengths),
                "s" if sum(lengths)>1 else "")
        else:
            getter = None
            if element.empty():
                getter = lambda: []
            elif element.is_int():
                getter = element.as_int
            elif element.is_real():
                getter = element.as_real
            elif element.is_string():
                getter = lambda: [
                    odil.as_unicode(x, specific_character_set)
                    for x in element.as_string()]
            value = [x for x in getter()]

            if decode_uids and element.vr == odil.VR.UI:
                value = [
                    odil.registry.uids_dictionary[uid].name
                        if uid in odil.registry.uids_dictionary else uid
                    for uid in value
                ]

        print("{}{}{} {:04x},{:04x} {} {}".format(
            padding,
            name, (max_length-len(name)-len(padding))*" ",
            tag.group, tag.element, element.vr,
            value))

        if element.is_data_set():
            sequence = element.as_data_set()
            if sequence:
                for item in sequence[:-1]:
                    print_data_set(
                        item, decode_uids, padding+"  ", max_length,
                        specific_character_set)
                    print()
                print_data_set(
                    sequence[-1], decode_uids, padding+"  ", max_length,
                    specific_character_set)

def find_max_name_length(data_set, max_length=0, padding_length=0):
    for tag, element in data_set.items():
        if tag in odil.registry.public_dictionary:
            entry = odil.registry.public_dictionary[tag]
            length = len(entry.name)
        else:
            length = 9 # xxxx,yyyy
        max_length = max(max_length, padding_length+length)
        if element.is_data_set():
            sequence = element.as_data_set()
            for item in sequence:
                max_length = max(
                    max_length,
                    find_max_name_length(item, max_length, 2+padding_length))
    return max_length



import odil

association = odil.Association()
association.set_peer_host("184.73.255.26")
association.set_peer_port(11112)

presentation_contexts = [
    odil.AssociationParameters.PresentationContext(
        1, odil.registry.StudyRootQueryRetrieveInformationModelFind,
        [ odil.registry.ExplicitVRLittleEndian ], True, False
    ),
    odil.AssociationParameters.PresentationContext(
        3, odil.registry.Verification,
        [ odil.registry.ExplicitVRLittleEndian ], True, False
    )
]

association.update_parameters()\
    .set_calling_ae_title("myself")\
    .set_called_ae_title("AWSPIXELMEDPUB") \
    .set_presentation_contexts(presentation_contexts)
association.associate()

query = odil.DataSet()
query.add(odil.registry.PatientName, odil.Value.Strings(["*"]))
query.add(odil.registry.QueryRetrieveLevel, odil.Value.Strings(["STUDY"]))
query.add(odil.registry.StudyDescription)
query.add(odil.registry.StudyDate)

find = odil.FindSCU(association)
find.set_affected_sop_class(odil.registry.StudyRootQueryRetrieveInformationModelFind)

def callback(data_set):
    print data_set.as_string(odil.registry.PatientName)[0]
find.find(query, callback)

data_sets = find.find(query)
print len(data_sets), "found"
for data_set in data_sets:
    print data_set.as_string(odil.registry.PatientName)[0]



import unittest

import odil

class TestURL(unittest.TestCase):
    def test_parse(self):
        url = odil.webservices.URL.parse("http://foo.com/bar?plip=plop#here")
        self.assertEqual(url.scheme, "http")
        self.assertEqual(url.authority, "foo.com")
        self.assertEqual(url.path, "/bar")
        self.assertEqual(url.query, "plip=plop")
        self.assertEqual(url.fragment, "here")

    def test_default_arguments(self):
        url = odil.webservices.URL("http", "foo.com", "/bar", fragment="here")
        self.assertEqual(url.scheme, "http")
        self.assertEqual(url.authority, "foo.com")
        self.assertEqual(url.path, "/bar")
        self.assertEqual(url.query, "")
        self.assertEqual(url.fragment, "here")

    def test_serialize(self):
        url = odil.webservices.URL("http", "foo.com", "/bar", "plip=plop", "here")
        self.assertEqual(str(url), "http://foo.com/bar?plip=plop#here")

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestElementsDictionaryKey(unittest.TestCase):
    def test_empty_constructor(self):
        key = odil.ElementsDictionaryKey()
    
    def test_tag_constructor(self):
        key = odil.ElementsDictionaryKey(odil.registry.PatientName)
    
    def test_string_constructor(self):
        key = odil.ElementsDictionaryKey("60xx0010")

class TestElementsDictionaryEntry(unittest.TestCase):
    def test_constructor(self):
        entry = odil.ElementsDictionaryEntry(
            "Patient's Name", "PatientName", "PN", "1")
        self.assertEqual(entry.name, "Patient's Name")
        self.assertEqual(entry.keyword, "PatientName")
        self.assertEqual(entry.vr, "PN")
        self.assertEqual(entry.vm, "1")

class TestElementsDictionary(unittest.TestCase):
    def test_contains(self):
        self.assertTrue(
            odil.registry.PatientName in odil.registry.public_dictionary)
        self.assertFalse(
            odil.Tag(0x0001, 0x0001) in odil.registry.public_dictionary)
        self.assertTrue("60xx0010" in odil.registry.public_dictionary)
        self.assertFalse("foo" in odil.registry.public_dictionary)
    
    def test_get_item_tag(self):
        entry = odil.registry.public_dictionary[odil.registry.PatientName]
        self.assertEqual(entry.name, "Patient's Name")
        self.assertEqual(entry.keyword, "PatientName")
        self.assertEqual(entry.vr, "PN")
        self.assertEqual(entry.vm, "1")
    
    def test_get_item_string(self):
        entry = odil.registry.public_dictionary["60xx0010"]
        self.assertEqual(entry.name, "Overlay Rows")
        self.assertEqual(entry.keyword, "OverlayRows")
        self.assertEqual(entry.vr, "US")
        self.assertEqual(entry.vm, "1")

if __name__ == "__main__":
    unittest.main()



import glob
import multiprocessing
import os
import subprocess
import sys
import time
import unittest

if sys.platform == "darwin" and multiprocessing.get_start_method(True) is None:
    multiprocessing.set_start_method("fork")

import odil

class Generator(odil.GetSCP.DataSetGenerator):
    def __init__(self):
        odil.GetSCP.DataSetGenerator.__init__(self)
        self._responses = []
        self._response_index = None
        
    def initialize(self, message):
        data_set_1 = odil.DataSet()
        data_set_1.add("SOPClassUID", [odil.registry.RawDataStorage])
        data_set_1.add(
            "SOPInstanceUID", 
            ["1.2.826.0.1.3680043.9.5560.3127449359877365688774406533090568532"])
        data_set_1.add("PatientName", ["Hello^World"])
        data_set_1.add("PatientID", ["1234"])
        self._responses.append(data_set_1)
        
        data_set_2 = odil.DataSet()
        data_set_2.add("SOPClassUID", [odil.registry.RawDataStorage])
        data_set_2.add(
            "SOPInstanceUID", 
            ["1.2.826.0.1.3680043.9.5560.3221615743193123463515381981101110692"])
        data_set_2.add("PatientName", ["Doe^John"])
        data_set_2.add("PatientID", ["5678"])
        self._responses.append(data_set_2)
        
        self._response_index = 0
    
    def done(self):
        return (self._response_index == len(self._responses))
    
    def next(self):
        self._response_index += 1
    
    def get(self):
        return self._responses[self._response_index]
    
    def count(self):
        return 2

class TestGetSCP(unittest.TestCase):
    def test_get_scp_release(self):
        process = multiprocessing.Process(target=self.run_server)
        process.start()
        time.sleep(0.5)
        data_sets = self.run_client()
        process.join(2)
        server_code = process.exitcode
        
        self.assertEqual(server_code, 0)
        
        self.assertEqual(len(data_sets), 2)
        
        self.assertEqual(len(data_sets[0]), 4)
        
        self.assertSequenceEqual(
            data_sets[0].as_string("SOPClassUID"), 
            [odil.registry.RawDataStorage])
        self.assertSequenceEqual(
            data_sets[0].as_string("SOPInstanceUID"), 
            [b"1.2.826.0.1.3680043.9.5560.3127449359877365688774406533090568532"])
        self.assertSequenceEqual(
            data_sets[0].as_string("PatientName"), [b"Hello^World"])
        self.assertSequenceEqual(data_sets[0].as_string("PatientID"), [b"1234"])
        
        self.assertSequenceEqual(
            data_sets[1].as_string("SOPClassUID"), 
            [odil.registry.RawDataStorage])
        self.assertSequenceEqual(
            data_sets[1].as_string("SOPInstanceUID"), 
            [b"1.2.826.0.1.3680043.9.5560.3221615743193123463515381981101110692"])
        self.assertSequenceEqual(
            data_sets[1].as_string("PatientName"), [b"Doe^John"])
        self.assertSequenceEqual(data_sets[1].as_string("PatientID"), [b"5678"])
    
    def run_client(self):
        command = [
            "getscu", 
            "-ll", "error",
            "-P", "-k", "QueryRetrieveLevel=PATIENT",
            "-k", "PatientID=*", "-k", "PatientName",
            "+B",
            "localhost", "11113"]
        
        retcode = subprocess.call(command)
        if retcode != 0:
            return []
        
        files = sorted(glob.glob("{}*".format(odil.uid_prefix)))
        data_sets = []
        for file_ in files:
            with odil.open(file_, "rb") as fd:
                data_sets.append(odil.Reader.read_file(fd)[1])
        for file_ in files:
            os.remove(file_)
        
        return data_sets

    def run_server(self):
        association = odil.Association()
        association.set_tcp_timeout(1)
        association.receive_association("v4", 11113)

        get_scp = odil.GetSCP(association)
        generator = Generator()
        get_scp.set_generator(generator)

        message = association.receive_message()
        get_scp(message)
        
        termination_ok = False

        try:
            association.receive_message()
        except odil.AssociationReleased:
            termination_ok = True
        except odil.AssociationAborted:
            pass
        
        if termination_ok:
            sys.exit(0)
        else:
            sys.exit(1)

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestException(unittest.TestCase):
    def test_python_generic(self):
        with self.assertRaises(Exception):
            raise odil.Exception()
    
    def test_python_specific(self):
        with self.assertRaises(odil.Exception):
            raise odil.Exception()

    def test_cpp_generic(self):
        tag = odil.Tag(0xdead, 0xbeef)
        with self.assertRaises(Exception):
            tag.get_name()
    
    def test_cpp_specific(self):
        tag = odil.Tag(0xdead, 0xbeef)
        with self.assertRaises(odil.Exception):
            tag.get_name()

if __name__ == "__main__":
    unittest.main()



import logging
import os

import odil

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "dicomdir",
        help="Create a DICOMDIR")
    parser.add_argument(
        "names", nargs="+", metavar="name", help="DICOM files or directories")
    parser.add_argument(
        "directory", help="Output directory for the DICOMDIR file")
    parser.add_argument(
        "--patient-key", "-p", default=[], action="append",
        help="User-defined keys for PATIENT-level records, "
            "expressed as KEYWORD[:TYPE]. TYPE defaults to 3.")
    parser.add_argument(
        "--study-key", "-S", default=[], action="append",
        help="User-defined keys for STUDY-level records")
    parser.add_argument(
        "--series-key", "-s", default=[], action="append", 
        help="User-defined keys for SERIES-level records")
    parser.add_argument(
        "--image-key", "-i", default=[], action="append",
        help="User-defined keys for IMAGE-level records")
    parser.set_defaults(function=create_dicomdir)
    return parser

def create_dicomdir(
        names, directory, patient_key, study_key, series_key, image_key):
    files = []
    for name in names:
        if os.path.isfile(name):
            files.append(os.path.abspath(name))
        elif os.path.isdir(name):
            for dirpath, _, filenames in os.walk(name):
                dirpath = os.path.abspath(dirpath)
                filenames = [os.path.join(dirpath, x) for x in filenames]
                dicom_files = []
                for filename in filenames:
                    try:
                        odil.Reader.read_file(
                            filename, halt_condition=lambda x: True)
                    except:
                        # Not a DICOM file
                        pass
                    else:
                        dicom_files.append(filename)
                files.extend(dicom_files)
    
    directory = os.path.abspath(directory)
    if not all(x.startswith(directory) for x in files):
        raise Exception("All files must be under {}".format(directory))
    
    files = [x[len(directory)+1:] for x in files]
    
    keys = {}
    for level in ["patient", "study", "series", "image"]:
        for entry in locals()["{}_key".format(level)]:
            if ":" in entry:
                tag, type_ = entry.split(":")
            else:
                tag, type_ = entry, "3"
            tag = getattr(odil.registry, tag)
            type_ = int(type_)
            keys.setdefault(level.upper(), []).append((tag, type_))
    
    creator = odil.BasicDirectoryCreator(directory, files, keys)
    creator()



import os
import sys
import unittest

import odil

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from peer_fixture_base import PeerFixtureBase

class TestStoreSCU(PeerFixtureBase):
    def setUp(self):
        PeerFixtureBase.setUp(
            self, 
            [
                odil.AssociationParameters.PresentationContext(
                    1, odil.registry.RawDataStorage,
                    [ odil.registry.ImplicitVRLittleEndian ], 
                    odil.AssociationParameters.PresentationContext.Role.SCU
                )
            ])

        self.data_set = odil.DataSet()
        self.data_set.add("ImageType", ["ORIGINAL", "PRIMARY", "OTHER"])
        self.data_set.add("PatientID", ["1234"])
        self.data_set.add(
            "StudyInstanceUID",
            ["2.25.386726390606491051215227596277040710"])
        self.data_set.add(
            "SeriesInstanceUID",
            ["2.25.235367796740370588607388995952651763168"])
        self.data_set.add("SOPClassUID", [odil.registry.RawDataStorage])
        self.data_set.add(
            "SOPInstanceUID",
            ["2.25.294312554735929033890522327215919068328"])
    
    def test_affected_sop_class_uid(self):
        store = odil.StoreSCU(self.association)
        store.set_affected_sop_class(self.data_set)
        self.assertEqual(
            store.get_affected_sop_class().encode(),
            odil.registry.RawDataStorage)
    
    def test_store(self):
        store = odil.StoreSCU(self.association)

        store.set_affected_sop_class(self.data_set)
        store.store(self.data_set)

if __name__ == "__main__":
    unittest.main()



import contextlib
import warnings

from ._odil import *

_open = open

@contextlib.contextmanager
def open(path, mode="rb"):
    fd = _open(path, mode)
    stream = iostream(fd)
    try:
        yield stream
    finally:
        fd.close()

def read(path, keep_group_length=False, halt_condition=None):
    warnings.warn("odil.read is deprecated. Use odil.open instead")
    with open(path) as stream:
        return Reader.read_file(stream, keep_group_length, halt_condition)

def write(
        data_set, path, meta_information=DataSet(),
        transfer_syntax=registry.ExplicitVRLittleEndian,
        item_encoding=Writer.ItemEncoding.ExplicitLength,
        use_group_length=False):
    warnings.warn("odil.write is deprecated. Use odil.open instead")
    with open(path, "wb") as stream:
        Writer.write_file(
            data_set, stream, meta_information, transfer_syntax, item_encoding,
            use_group_length)



import os
import tempfile
import unittest

from io import BytesIO

import odil

class TestReader(unittest.TestCase):
    def test_constructor(self):
        stream = odil.iostream(BytesIO())
        reader = odil.Reader(stream, odil.registry.ImplicitVRLittleEndian)
        self.assertEqual(
            reader.transfer_syntax.encode(),
            odil.registry.ImplicitVRLittleEndian)
        self.assertEqual(reader.byte_ordering, odil.ByteOrdering.LittleEndian)
        self.assertFalse(reader.explicit_vr)
        self.assertFalse(reader.keep_group_length)

    def test_constructor_no_default(self):
        stream = odil.iostream(BytesIO())
        reader = odil.Reader(stream, odil.registry.ExplicitVRBigEndian, True)
        self.assertEqual(
            reader.transfer_syntax.encode(), odil.registry.ExplicitVRBigEndian)
        self.assertEqual(reader.byte_ordering, odil.ByteOrdering.BigEndian)
        self.assertTrue(reader.explicit_vr)
        self.assertTrue(reader.keep_group_length)

    def test_read_data_set(self):
        string_io = BytesIO(
            b"\x10\x00\x10\x00PN\x07\x00Foo^Bar"
            b"\x10\x00\x20\x00CS\x03\x00FOO"
        )
        stream = odil.iostream(string_io)
        reader = odil.Reader(stream, odil.registry.ExplicitVRLittleEndian)
        data_set = reader.read_data_set()
        self.assertEqual(data_set.size(), 2)
        self.assertSequenceEqual(data_set.as_string("PatientName"), [b"Foo^Bar"])
        self.assertSequenceEqual(data_set.as_string("PatientID"), [b"FOO"])

    def test_read_data_set_halt_condition(self):
        string_io = BytesIO(
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar "
            b"\x10\x00\x20\x00" b"LO" b"\x04\x00" b"FOO "
        )
        stream = odil.iostream(string_io)
        reader = odil.Reader(stream, odil.registry.ExplicitVRLittleEndian)
        data_set = reader.read_data_set(lambda x: x==odil.registry.PatientID)
        self.assertEqual(data_set.size(), 1)
        self.assertSequenceEqual(data_set.as_string("PatientName"), [b"Foo^Bar"])

    def test_read_tag(self):
        string_io = BytesIO(b"\x10\x00\x20\x00")
        stream = odil.iostream(string_io)
        reader = odil.Reader(stream, odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(reader.read_tag(), odil.registry.PatientID)

    def test_read_length(self):
        string_io = BytesIO(b"\x34\x12")
        stream = odil.iostream(string_io)
        reader = odil.Reader(stream, odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(reader.read_length(odil.VR.CS), 0x1234)

    def test_read_element(self):
        string_io = BytesIO(b"PN\x08\x00Foo^Bar ")
        stream = odil.iostream(string_io)
        reader = odil.Reader(stream, odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(
            reader.read_element(odil.registry.PatientName),
            odil.Element([b"Foo^Bar"], odil.VR.PN))

    def test_read_file_stream(self):
        data = (
            128*b"\x00"+b"DICM"+
            b"\x02\x00\x10\x00" b"UI" b"\x14\x00" b"1.2.840.10008.1.2.1\x00"
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar "
        )
        string_io = BytesIO(data)
        stream = odil.iostream(string_io)

        header, data_set = odil.Reader.read_file(stream)

        self.assertEqual(len(header), 1)
        self.assertSequenceEqual(
            header.as_string("TransferSyntaxUID"),
            [odil.registry.ExplicitVRLittleEndian])

        self.assertEqual(len(data_set), 1)
        self.assertSequenceEqual(data_set.as_string("PatientName"), [b"Foo^Bar"])
    
    def test_read_file_path(self):
        data = (
            128*b"\x00"+b"DICM"+
            b"\x02\x00\x10\x00" b"UI" b"\x14\x00" b"1.2.840.10008.1.2.1\x00"
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar "
        )
        
        fd, path = tempfile.mkstemp()
        os.write(fd, data)
        os.close(fd)
        
        try:
            header, data_set = odil.Reader.read_file(path)
        finally:
            os.remove(path)

        self.assertEqual(len(header), 1)
        self.assertSequenceEqual(
            header.as_string("TransferSyntaxUID"),
            [odil.registry.ExplicitVRLittleEndian])

        self.assertEqual(len(data_set), 1)
        self.assertSequenceEqual(data_set.as_string("PatientName"), [b"Foo^Bar"])
    
    def test_open_context(self):
        data = (
            128*b"\x00"+b"DICM"+
            b"\x02\x00\x10\x00" b"UI" b"\x14\x00" b"1.2.840.10008.1.2.1\x00"
            b"\x10\x00\x10\x00" b"PN" b"\x08\x00" b"Foo^Bar "
        )
    
        fd, path = tempfile.mkstemp()
        os.write(fd, data)
        os.close(fd)
    
        try:
            with odil.open(path) as fd:
                header, data_set = odil.Reader.read_file(fd)
        finally:
            os.remove(path)

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestMessage(unittest.TestCase):
    def test_default_constructor(self):
        message = odil.webservices.Message()
        self.assertEqual(dict(message.get_headers()), {})
        self.assertEqual(message.get_body(), b"")

    def test_full_constructor(self):
        message = odil.webservices.Message(headers={"foo": "bar"}, body="body")
        self.assertEqual(dict(message.get_headers()), {"foo": "bar"})
        self.assertEqual(message.get_body(), b"body")

    def test_existing_header(self):
        message = odil.webservices.Message({"foo": "bar", "plip": "plop"})
        self.assertTrue(message.has_header("foo"))
        self.assertEqual(message.get_header("foo"), "bar")

    def test_missing_header(self):
        message = odil.webservices.Message({"foo": "bar", "plip": "plop"})
        self.assertFalse(message.has_header("none"))
        with self.assertRaises(odil.Exception):
            message.get_header("none")

    def test_set_headers(self):
        message = odil.webservices.Message()
        message.set_headers({"foo": "bar", "plip": "plop"})
        self.assertEqual(
            dict(message.get_headers()), {"foo": "bar", "plip": "plop"})

    def test_modify_header(self):
        message = odil.webservices.Message({"foo": "bar", "plip": "plop"})
        message.set_header("foo", "baz")
        self.assertEqual(message.get_header("foo"), "baz")

    def test_add_header(self):
        message = odil.webservices.Message()
        message.set_header("foo", "baz")
        self.assertEqual(message.get_header("foo"), "baz")

    def test_body(self):
        message = odil.webservices.Message()
        message.set_body("body")
        self.assertEqual(message.get_body(), b"body")

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestSelector(unittest.TestCase):
    def test_default_constructor(self):
        selector = odil.webservices.Selector()
        self.assertFalse(selector.is_study_present())
        self.assertFalse(selector.is_series_present())
        self.assertFalse(selector.is_instance_present())
        self.assertEqual(list(selector.get_frames()), [])

    def test_full_constructor(self):
        selector = odil.webservices.Selector({"studies" :"1.2", "series" :"3.4", "instances" :"5.6"}, [7,8])
        self.assertTrue(selector.is_study_present())
        self.assertEqual(selector.get_study(), "1.2")

        self.assertTrue(selector.is_series_present())
        self.assertEqual(selector.get_series(), "3.4")

        self.assertTrue(selector.is_instance_present())
        self.assertEqual(selector.get_instance(), "5.6")

        self.assertEqual(list(selector.get_frames()), [7,8])

    def test_get_path_no_frames(self):
        selector = odil.webservices.Selector(
            {"studies" :"1.2", "series" :"3.4", "instances" :"5.6"}, [7,8])
        self.assertEqual(
            selector.get_path(False), "/studies/1.2/series/3.4/instances/5.6")

    def test_get_path_frames(self):
        selector = odil.webservices.Selector(
            {"studies" :"1.2", "series" :"3.4", "instances" :"5.6"}, [7,8])
        self.assertEqual(
            selector.get_path(True),
            "/studies/1.2/series/3.4/instances/5.6/frames/7,8")

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestWADORSRequest(unittest.TestCase):
    def test_default_constructor(self):
        request = odil.webservices.WADORSRequest()
        self.assertEqual(str(request.get_base_url()), "")
        self.assertEqual(request.get_transfer_syntax(), "")
        self.assertEqual(request.get_character_set(), "")
        self.assertFalse(request.get_include_media_type_in_query())
        self.assertFalse(request.get_include_character_set_in_query())

    def test_http_request_constructor(self):
        http_request = odil.webservices.HTTPRequest(
            "GET", odil.webservices.URL.parse("/foo/studies/1.2"),
            headers={"Accept": "multipart/related;type=application/dicom"})
        wado_rs_request = odil.webservices.WADORSRequest(http_request)

        self.assertEqual(str(wado_rs_request.get_base_url()), "/foo")
        self.assertEqual(
            wado_rs_request.get_transfer_syntax().encode(),
            odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(wado_rs_request.get_character_set(), "UTF-8")
        self.assertFalse(wado_rs_request.get_include_media_type_in_query())
        self.assertFalse(wado_rs_request.get_include_character_set_in_query())

        self.assertEqual(
            wado_rs_request.get_type(), odil.webservices.Utils.Type.DICOM)
        self.assertEqual(
            wado_rs_request.get_selector(),
            odil.webservices.Selector({"studies" : "1.2"}))
        self.assertEqual(str(wado_rs_request.get_url()), "/foo/studies/1.2")
        self.assertEqual(wado_rs_request.get_media_type(), "application/dicom")
        self.assertEqual(
            wado_rs_request.get_representation(),
            odil.webservices.Utils.Representation.DICOM)

    def test_full_constructor(self):
        request = odil.webservices.WADORSRequest(
            base_url=odil.webservices.URL.parse("http://foo.com/bar"),
            transfer_syntax=odil.registry.ExplicitVRLittleEndian,
            character_set="utf-8", include_media_type_in_query=True,
            include_character_set_in_query=True)
        self.assertEqual(str(request.get_base_url()), "http://foo.com/bar")
        self.assertEqual(
            request.get_transfer_syntax().encode(),
            odil.registry.ExplicitVRLittleEndian)
        self.assertEqual(request.get_character_set(), "utf-8")
        self.assertTrue(request.get_include_media_type_in_query())
        self.assertTrue(request.get_include_character_set_in_query())

if __name__ == "__main__":
    unittest.main()



import re
import unittest

import odil

class TestJSONConverter(unittest.TestCase):
    def setUp(self):
        self.data_set = odil.DataSet()
        self.data_set.add("PatientName", ["Foo^Bar"])

        self.json = """{"00100010":{"Value":[{"Alphabetic":"Foo^Bar"}],"vr":"PN"}}"""

    def test_ugly_print(self):
        serialized = odil.as_json(self.data_set, False)
        self.assertEqual(re.sub(r"\s+", "", serialized), self.json)

    def test_pretty_print(self):
        serialized = odil.as_json(self.data_set, True)
        self.assertEqual(re.sub(r"\s+", "", serialized), self.json)

    def test_as_dataset(self):
        data_set = odil.from_json(self.json)
        self.assertTrue(data_set == self.data_set)

if __name__ == "__main__":
    unittest.main()



import argparse
import logging

import odil

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "transcode",
        help="Change the transfer syntax or the format",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("input", help="Input file name")
    parser.add_argument("output", help="Output file name")
    parser.add_argument(
        "--format", "-f",
        choices=["binary", "json", "xml"], default="binary",
        help="Output file format")
    parser.add_argument(
        "--transfer-syntax", "-t",
        help="Transfer syntax of output file (binary only)")
    parser.add_argument(
        "--pretty-print", "-p", action="store_true",
        help="Pretty-print in output file (JSON and XML only)")
    parser.set_defaults(function=transcode)
    return parser

def transcode(input, output, format, transfer_syntax, pretty_print):

    kwargs = {}

    if format == "binary":
        if transfer_syntax:
            transfer_syntax = getattr(odil.registry, transfer_syntax)
            ts_info = odil.registry.uids_dictionary[transfer_syntax]
            if ts_info.type != "Transfer Syntax":
                raise Exception(
                    "{} is not a transfer syntax".format(ts_info.name))
            kwargs["transfer_syntax"] = transfer_syntax
        else:
            kwargs["transfer_syntax"] = None
    elif format == "json":
        kwargs["pretty_print"] = pretty_print
    elif format == "xml":
        kwargs["pretty_print"] = pretty_print

    globals()["as_{}".format(format)](input, output, **kwargs)

def as_binary(input, output, transfer_syntax):
    with odil.open(input) as stream:
        _, data_set = odil.Reader.read_file(stream)
    with odil.open(output, "wb") as fd:
        odil.Writer.write_file(data_set, fd, transfer_syntax=transfer_syntax)

def as_json(input, output, pretty_print):
    with odil.open(input) as stream:
        _, data_set = odil.Reader.read_file(stream)
    with open(output, "w") as fd:
        json = odil.as_json(data_set, pretty_print)
        fd.write(json)

def as_xml(input, output, pretty_print):
    with odil.open(input) as stream:
        _, data_set = odil.Reader.read_file(stream)
    with open(output, "w") as fd:
        xml = odil.as_xml(data_set, pretty_print)
        fd.write(xml)



import argparse
import logging

import odil

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "echo", help="Ping a remote DICOM server (C-ECHO)",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("host", help="Remote host address")
    parser.add_argument("port", type=int, help="Remote host port")
    parser.add_argument(
        "calling_ae_title", help="AE title of the calling application")
    parser.add_argument(
        "called_ae_title", help="AE title of the called application")
    parser.set_defaults(function=echo)
    return parser

def echo(host, port, calling_ae_title, called_ae_title):
    association = odil.Association()
    association.set_peer_host(host)
    association.set_peer_port(port)
    association.update_parameters()\
        .set_calling_ae_title(calling_ae_title)\
        .set_called_ae_title(called_ae_title) \
        .set_presentation_contexts([
            odil.AssociationParameters.PresentationContext(
                3, odil.registry.Verification,
                [ odil.registry.ImplicitVRLittleEndian ], 
                odil.AssociationParameters.PresentationContext.Role.SCU
            )
        ])
    association.associate()
    logging.info("Association established")

    echo = odil.EchoSCU(association)
    echo.echo()
    logging.info("C-ECHO successful")

    association.release()
    logging.info("Association released")



import pickle
import unittest

import odil

class TestTag(unittest.TestCase):
    def test_two_ints_constructor(self):
        tag = odil.Tag(0x1234, 0x5678)
        self.assertEqual(tag.group, 0x1234)
        self.assertEqual(tag.element, 0x5678)

    def test_one_int_constructor(self):
        tag = odil.Tag(0x12345678)
        self.assertEqual(tag.group, 0x1234)
        self.assertEqual(tag.element, 0x5678)

    def test_string_constructor(self):
        tag = odil.Tag("PatientID")
        self.assertEqual(tag.group, 0x0010)
        self.assertEqual(tag.element, 0x0020)

    def test_group(self):
        tag = odil.Tag(0x1234, 0x5678)
        tag.group = 0x4321
        self.assertEqual(tag.group, 0x4321)
        self.assertEqual(tag.element, 0x5678)

    def test_element(self):
        tag = odil.Tag(0x1234, 0x5678)
        tag.element = 0x8765
        self.assertEqual(tag.group, 0x1234)
        self.assertEqual(tag.element, 0x8765)

    def test_is_private(self):
        public = odil.Tag(0x1234, 0x5678)
        self.assertTrue(not public.is_private())

        private = odil.Tag(0x1235, 0x5678)
        self.assertTrue(private.is_private())

    def test_get_name(self):
        tag = odil.Tag(0x0010, 0x0020)
        self.assertEqual(tag.get_name(), "PatientID")

    def test_equality(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertTrue(tag1 == tag1)
        self.assertFalse(tag1 == tag2)

    def test_inequality(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertFalse(tag1 != tag1)
        self.assertTrue(tag1 != tag2)

    def test_less(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertTrue(tag1 < tag2)
        self.assertFalse(tag2 < tag1)

    def test_greater(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertFalse(tag1 > tag2)
        self.assertTrue(tag2 > tag1)

    def test_less_or_equal(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertTrue(tag1 <= tag2)
        self.assertTrue(tag1 <= tag1)
        self.assertFalse(tag2 <= tag1)

    def test_greater_or_equal(self):
        tag1 = odil.Tag(0x1234, 0x5678)
        tag2 = odil.Tag(0x4321, 0x8765)
        self.assertFalse(tag1 >= tag2)
        self.assertTrue(tag1 >= tag1)
        self.assertTrue(tag2 >= tag1)

    def test_str(self):
        tag = odil.Tag(0x1234, 0x5678)
        self.assertEqual(str(tag), "12345678")

    def test_hash(self):
        tag = odil.Tag(
            odil.registry.PatientName.group, odil.registry.PatientName.element)
        d = {tag: True}

        self.assertTrue(tag in d)
        self.assertTrue(odil.registry.PatientName in d)

        other = odil.Tag(
            odil.registry.PatientID.group, odil.registry.PatientID.element)
    
    def test_pickle(self):
        self.assertEqual(
            pickle.loads(pickle.dumps(odil.registry.PatientName)),
            odil.registry.PatientName)

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestMessage(unittest.TestCase):
    def test_commands(self):
        self.assertEqual(odil.messages.Message.Command.C_FIND_RSP, 0x8020)

if __name__ == "__main__":
    unittest.main()



import re
import unittest

import odil

class TestXMLConverter(unittest.TestCase):
    def setUp(self):
        self.data_set = odil.DataSet()
        self.data_set.add("PatientName", ["Foo^Bar"])

        self.xml = (
            """<?xml version="1.0" encoding="utf-8"?>"""
              "<NativeDicomModel>"
                """<DicomAttribute vr="PN" tag="00100010" keyword="PatientName">"""
                  """<PersonName number="1">"""
                    "<Alphabetic>"
                      "<FamilyName>Foo</FamilyName>"
                      "<GivenName>Bar</GivenName>"
                    "</Alphabetic>"
                  "</PersonName>"
                "</DicomAttribute>"
              "</NativeDicomModel>"
            )

    def test_ugly_print(self):
        serialized = odil.as_xml(self.data_set, False)
        self.assertEqual(re.sub(r"\n+", "", serialized), self.xml)

    def test_pretty_print(self):
        serialized = odil.as_xml(self.data_set, True)
        serialized = re.sub(r"\n\s+", "\n", serialized, re.M)
        serialized = re.sub(r"\n", "", serialized)
        self.assertEqual(serialized, self.xml)

    def test_as_dataset(self):
        data_set = odil.from_xml(self.xml)
        self.assertTrue(data_set == self.data_set)

if __name__ == "__main__":
    unittest.main()



import email
import sys
import unittest

import odil


class TestSTOWRSRequest(unittest.TestCase):
    base_url_http = odil.webservices.URL("http", "example.com", "/dicom", "", "")
    full_url = odil.webservices.URL(
        "http", # scheme
        "example.com", # authority
        "/dicom/studies/1.2", # path
        "", # query
        "" # fragment
    )

    selector = odil.webservices.Selector({"studies" : "1.2"})

    def setUp(self):
        data_set_1 = odil.DataSet()
        data_set_1.add("SOPClassUID", [odil.registry.RawDataStorage])
        data_set_1.add("SOPInstanceUID", ["1.2.3.4"])
        data_set_1.add("PatientID", ["DJ1234"])
        data_set_1.add("PixelSpacing", [1.5, 2.5])

        data_set_2 = odil.DataSet()
        data_set_2.add("SOPClassUID", [odil.registry.MRImageStorage])
        data_set_2.add("SOPInstanceUID", ["1.2.3.5"])
        data_set_2.add("PatientName", ["Doe^John"])
        data_set_2.add("PatientAge", ["042Y"])

        bin = odil.Value.BinaryItem()
        bin.append(0x01)
        bin.append(0x02)
        bin.append(0x03)
        bin.append(0x04)
        binary = odil.Value.Binary()
        binary.append(bin)
        # Fix Binary factory with BinaryItem
        data_set_2.add("Signature", [bytearray([0x01, 0x02, 0x03, 0x04])])

        self.data_sets = [data_set_1, data_set_2]

    def test_constructor_baseUrl(self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        self.assertEqual(request.get_base_url(), self.base_url_http)

    def test_base_url(self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        base_url_http2 = odil.webservices.URL("http", "second_example.com", "/dicom", "", "")
        request.set_base_url(base_url_http2)
        self.assertEqual(request.get_base_url(), base_url_http2)

    def test_constructor_http(self):
        http_request = odil.webservices.HTTPRequest("POST", self.full_url)
        http_request.set_header("Content-Type", "multipart/related;type=application/dicom")
        request = odil.webservices.STOWRSRequest(http_request)

        # Check for full_url
        self.assertEqual(request.get_url(), self.full_url)

        # Check for media type
        self.assertEqual(request.get_media_type(), "application/dicom")

        # Check for representation
        self.assertEqual(request.get_representation(), odil.webservices.Utils.Representation.DICOM)

        # Check for selector
        self.assertEqual(request.get_selector(), self.selector)

    def test_fail_request_datasets(self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        invalid_selector = odil.webservices.Selector({"studies" : "1.2", "series" : "3.4"})
        with self.assertRaises(odil.Exception):
            request.request_dicom(self.data_sets, invalid_selector, odil.webservices.Utils.Representation.DICOM)

    def test_get_http_request(self):

        #DICOM+XML
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        request.request_dicom(self.data_sets, self.selector, odil.webservices.Utils.Representation.DICOM_XML)
        http_request = request.get_http_request()
        # Check for http_request content
        self.assertTrue(http_request.has_header("Content-Type"))
        self.assertEqual(http_request.get_method(), "POST")
        self.assertEqual(http_request.get_http_version(), "HTTP/1.0")

        msg = self._http_message_to_email_message(http_request)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_param("type") != "application/dicom+xml":
                i = i+1
        self.assertEqual(i, 3) # 3 parts (One for each dataset and 1 for binary)
        self.assertEqual(request.get_base_url(), self.base_url_http)

        #DICOM
        request.request_dicom(self.data_sets, self.selector, odil.webservices.Utils.Representation.DICOM)
        http_request_dcm = request.get_http_request()
        msg = self._http_message_to_email_message(http_request_dcm)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_param("type") != "application/dicom":
                i = i+1
        self.assertEqual(i, 2) # 3 parts (One for each dataset and 1 for binary)

    def test_data_sets_equality(self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        request.request_dicom([self.data_sets[1]], self.selector, odil.webservices.Utils.Representation.DICOM_JSON)
        http_request = request.get_http_request()
        self.assertSequenceEqual(request.get_data_sets(), [self.data_sets[1]])

    def test_equality(self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        request.request_dicom(self.data_sets, self.selector, odil.webservices.Utils.Representation.DICOM)
        http_request = request.get_http_request()
        second_request = odil.webservices.STOWRSRequest(http_request)
        self.assertEqual(request, second_request)

    def test_equality_2 (self):
        request = odil.webservices.STOWRSRequest(self.base_url_http)
        request.request_dicom(self.data_sets, self.selector, odil.webservices.Utils.Representation.DICOM_XML)
        http_request = request.get_http_request()
        request_copy = odil.webservices.STOWRSRequest(http_request)
        self.assertEqual(request_copy, request)
        self.assertSequenceEqual(self.data_sets, request_copy.get_data_sets())

    def tearDown(self):
        self.data_sets = None

    def _http_message_to_email_message(self, http_message):
        message_bytes = [
            name.encode()+b": "+value.encode()
            for (name, value) in http_message.get_headers().items()]
        message_bytes.append(http_message.get_body())
        message_bytes = b"\r\n".join(message_bytes)

        if sys.version_info[0] >= 3:
            email_message = email.message_from_bytes(message_bytes)
        else:
            email_message = email.message_from_string(message_bytes)
        return email_message

if __name__ == "__main__":
    unittest.main()



from __future__ import print_function

import argparse

import odil

def add_subparser(subparsers):
    parser = subparsers.add_parser(
        "store", help="DICOM store (C-STORE)",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("host", help="Remote host address")
    parser.add_argument("port", type=int, help="Remote host port")
    parser.add_argument(
        "calling_ae_title", help="AE title of the calling application")
    parser.add_argument(
        "called_ae_title", help="AE title of the called application")
    parser.add_argument("filenames", nargs="+", help="File names")
    parser.set_defaults(function=store)
    return parser

def store(host, port, calling_ae_title, called_ae_title, filenames):
    transfer_syntaxes = [
        odil.registry.ExplicitVRLittleEndian,
        odil.registry.ImplicitVRLittleEndian,
    ]
    
    # Find all SOP classes to negotiate at association time. We don't need to
    # read the whole data set for this
    sop_classes = set()
    for filename in filenames:
        with odil.open(filename) as stream:
            _, data_set = odil.Reader.read_file(
                stream,
                halt_condition=lambda tag: tag>odil.registry.SOPClassUID)
        sop_classes.update(data_set.as_string("SOPClassUID"))

    presentation_contexts = [
        odil.AssociationParameters.PresentationContext(
            2*i+1, sop_class, transfer_syntaxes, 
            odil.AssociationParameters.PresentationContext.Role.SCU)
        for i, sop_class in enumerate(sop_classes)
    ]
    
    # Create the association and the Store SCU
    association = odil.Association()
    association.set_peer_host(host)
    association.set_peer_port(port)
    association.update_parameters()\
        .set_calling_ae_title(calling_ae_title)\
        .set_called_ae_title(called_ae_title)\
        .set_presentation_contexts(presentation_contexts)
    association.associate()
    
    negotiated_parameters = association.get_negotiated_parameters()
    negotiated_pc = negotiated_parameters.get_presentation_contexts()

    store = odil.StoreSCU(association)
    
    for filename in filenames:
        with odil.open(filename) as stream:
            _, data_set = odil.Reader.read_file(stream)
        
        try:
            store.set_affected_sop_class(data_set)
            store.store(data_set)
        except Exception as e:
            print("Could not store {}: {}".format(filename, e))
    
    association.release()



import unittest

import odil

class TestHTTPResponse(unittest.TestCase):
    def test_default_constructor(self):
        response = odil.webservices.HTTPResponse()
        self.assertEqual(response.get_http_version(), "")
        self.assertEqual(response.get_status(), 0)
        self.assertEqual(response.get_reason(), "")
        self.assertEqual(dict(response.get_headers()), {})
        self.assertEqual(response.get_body(), b"")

    def test_full_constructor(self):
        response = odil.webservices.HTTPResponse(
            http_version="HTTP/1.1", status=404, reason="not found",
            headers={"foo": "bar"}, body="body")
        self.assertEqual(response.get_http_version(), "HTTP/1.1")
        self.assertEqual(response.get_status(), 404)
        self.assertEqual(response.get_reason(), "not found")
        self.assertEqual(dict(response.get_headers()), {"foo":"bar"})
        self.assertEqual(response.get_body(), b"body")

    def test_http_version(self):
        response = odil.webservices.HTTPResponse()
        response.set_http_version("HTTP/1.1")
        self.assertEqual(response.get_http_version(), "HTTP/1.1")

    def test_status(self):
        response = odil.webservices.HTTPResponse()
        response.set_status(200)
        self.assertEqual(response.get_status(), 200)

    def test_reason(self):
        response = odil.webservices.HTTPResponse()
        response.set_reason("OK")
        self.assertEqual(response.get_reason(), "OK")

    def test_headers(self):
        response = odil.webservices.HTTPResponse()
        response.set_headers({"foo": "bar"})
        self.assertEqual(dict(response.get_headers()), {"foo": "bar"})

    def test_body(self):
        response = odil.webservices.HTTPResponse()
        response.set_body("body")
        self.assertEqual(response.get_body(), b"body")

if __name__ == "__main__":
    unittest.main()



import multiprocessing
import subprocess
import sys
import time
import unittest

if sys.platform == "darwin" and multiprocessing.get_start_method(True) is None:
    multiprocessing.set_start_method("fork")

import odil

class TestSCPDispatcher(unittest.TestCase):
    def test_echo(self):
        process = multiprocessing.Process(target=lambda: self.run_server(False))
        process.start()
        time.sleep(0.5)
        client_code = self.run_client()
        process.join(2)
        server_code = process.exitcode

        self.assertEqual(client_code, 0)
        self.assertEqual(server_code, 0)

    def run_client(self):
        command = ["echoscu", "-q"]
        command.extend(["localhost", "11113"])

        return subprocess.call(command)

    def run_server(self, use_abort):
        called = [False]
        def echo_callback(message):
            called[0] = True
            return 0

        association = odil.Association()
        association.set_tcp_timeout(1)
        association.receive_association("v4", 11113)

        echo_scp = odil.EchoSCP(association)
        echo_scp.set_callback(echo_callback)

        dispatcher = odil.SCPDispatcher(association)
        dispatcher.set_echo_scp(echo_scp)

        dispatcher.dispatch()

        termination_ok = False

        try:
            association.receive_message()
        except odil.AssociationReleased:
            termination_ok = True

        if called[0] and termination_ok:
            sys.exit(0)
        else:
            sys.exit(1)

if __name__ == "__main__":
    unittest.main()



import email
import io
import json
import tempfile
import sys
import unittest

import odil

class WADORSResponse(unittest.TestCase):
    def setUp(self):
        data_set_1 = odil.DataSet()
        data_set_1.add("SOPClassUID", [odil.registry.RawDataStorage])
        data_set_1.add("SOPInstanceUID", ["1.2.3.4"])
        data_set_1.add("PatientID", ["DJ1234"])
        data_set_1.add("PixelSpacing", [1.5, 2.5])

        data_set_2 = odil.DataSet()
        data_set_2.add("SOPClassUID", [odil.registry.MRImageStorage])
        data_set_2.add("SOPInstanceUID", ["1.2.3.5"])
        data_set_2.add("PatientName", ["Doe^John"])
        data_set_2.add("PatientAge", ["042Y"])

        self.data_sets =  [data_set_1, data_set_2]

        bin = odil.Value.BinaryItem()
        bin.append(0x01)
        bin.append(0x02)
        bulk_data_1 = odil.webservices.BulkData(bin, "foo/bar", "here")

        bin = odil.Value.BinaryItem()
        bin.append(0x03)
        bin.append(0x04)
        bulk_data_2 = odil.webservices.BulkData(bin, "plip/plop", "not/here")

        self.bulk_data = [
            bulk_data_1, bulk_data_2
        ]

    def test_constructor(self):
        response = odil.webservices.WADORSResponse()
        self.assertEqual(len(response.get_data_sets()), 0)
        self.assertEqual(len(response.get_bulk_data()), 0)
        self.assertFalse(response.is_partial())
        self.assertEqual(response.get_type(), odil.webservices.Utils.Type.None_)

    def test_data_sets(self):
        response = odil.webservices.WADORSResponse()
        response.set_data_sets(self.data_sets)
        self.assertSequenceEqual(response.get_data_sets(), self.data_sets)

    def test_bulk_data(self):
        response = odil.webservices.WADORSResponse()
        response.set_bulk_data(self.bulk_data)
        bulk = response.get_bulk_data()
        self.assertEqual(len(bulk), len(self.bulk_data))
        for i in range(len(bulk)):
            self.assertEqual(bulk[i].data.get_memory_view(), self.bulk_data[i].data.get_memory_view())
            self.assertEqual(bulk[i].type, self.bulk_data[i].type)
            self.assertEqual(bulk[i].location, self.bulk_data[i].location)

    def test_partial(self):
        response = odil.webservices.WADORSResponse()
        response.set_partial(True)
        self.assertTrue(response.is_partial())

    def test_respondFull(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_data_sets(self.data_sets)
        wado.set_partial(False)
        wado.respond_dicom(odil.webservices.Utils.Representation.DICOM)
        http = wado.get_http_response()
        self.assertEqual(http.get_status(), 200)
        self.assertEqual(http.get_reason(), "OK")

    def test_respondPartial(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_data_sets(self.data_sets)
        wado.set_partial(True)
        wado.respond_dicom(odil.webservices.Utils.Representation.DICOM)
        http = wado.get_http_response()
        self.assertEqual(http.get_status(), 206)
        self.assertEqual(http.get_reason(), "Partial Content")

    def test_respondDICOM(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_data_sets(self.data_sets)
        wado.respond_dicom(odil.webservices.Utils.Representation.DICOM)
        self.assertEqual(wado.get_type(), odil.webservices.Utils.Type.DICOM)
        self.assertEqual(wado.get_representation(), odil.webservices.Utils.Representation.DICOM)
        http = wado.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_content_type() == "application/dicom":
                file = tempfile.mkstemp()
                stream = io.FileIO(file[1], "wb")
                stream.write(part.get_payload(decode=True))
                stream.close()
                with odil.open(file[1], "rb") as stream:
                    ds = odil.Reader.read_file(stream)[1]
                self.assertEqual(self.data_sets[i], ds)
                i = i+1
        self.assertEqual(i, len(self.data_sets))

    def test_respondDICOMXML(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_data_sets(self.data_sets)
        wado.respond_dicom(odil.webservices.Utils.Representation.DICOM_XML)
        self.assertEqual(wado.get_type(), odil.webservices.Utils.Type.DICOM)
        self.assertEqual(wado.get_representation(), odil.webservices.Utils.Representation.DICOM_XML)
        http = wado.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_content_type() == "application/dicom+xml":
                self.assertEqual(odil.as_xml(self.data_sets[i], False), part.get_payload())
                i = i+1
        self.assertEqual(i, len(self.data_sets))

    def test_respondDICOMJSON(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_data_sets(self.data_sets)
        wado.respond_dicom(odil.webservices.Utils.Representation.DICOM_JSON)
        self.assertEqual(wado.get_type(), odil.webservices.Utils.Type.DICOM)
        self.assertEqual(wado.get_representation(), odil.webservices.Utils.Representation.DICOM_JSON)
        http = wado.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertFalse(msg.is_multipart())
        self.assertTrue(msg.get_content_type(), "application/dicom+json")
        odil_json = [json.loads(odil.as_json(x)) for x in self.data_sets]
        http_json = json.loads(http.get_body().decode())
        self.assertSequenceEqual(http_json, odil_json)

    def test_respondBulkData(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_bulk_data(self.bulk_data)
        wado.respond_bulk_data()
        self.assertEqual(wado.get_type(), odil.webservices.Utils.Type.BulkData)
        http = wado.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_param("type") != "application/octet-stream":
                self.assertEqual(part.get("Content-Location"), self.bulk_data[i].location)
                self.assertEqual(part.get_content_type(), self.bulk_data[i].type)
                self.assertEqual(
                    self.bulk_data[i].data.get_memory_view().tobytes(),
                    part.get_payload(decode=True))
                i = i+1
        self.assertEqual(i, len(self.bulk_data))

    def test_respondPixelData(self):
        wado = odil.webservices.WADORSResponse()
        wado.set_bulk_data(self.bulk_data)
        wado.respond_pixel_data("image/jpeg")
        self.assertEqual(wado.get_type(), odil.webservices.Utils.Type.PixelData)
        http = wado.get_http_response()

        msg = self._http_message_to_email_message(http)
        self.assertTrue(msg.is_multipart())

        i = 0
        for part in msg.walk():
            if part.get_param("type") != "image/jpeg":
                self.assertEqual(part.get("Content-Location"), self.bulk_data[i].location)
                self.assertEqual(part.get_content_type(), self.bulk_data[i].type)
                self.assertEqual(
                    self.bulk_data[i].data.get_memory_view().tobytes(),
                    part.get_payload(decode=True))
                i = i+1
        self.assertEqual(i, len(self.bulk_data))

    def tearDown(self):
        self.data_sets = None
        self.bulk_data = None

    def _http_message_to_email_message(self, http_message):
        message_bytes = [
            name.encode()+b": "+value.encode()
            for (name, value) in http_message.get_headers().items()]
        message_bytes.append(http_message.get_body())
        message_bytes = b"\r\n".join(message_bytes)

        if sys.version_info[0] >= 3:
            email_message = email.message_from_bytes(message_bytes)
        else:
            email_message = email.message_from_string(message_bytes)
        return email_message

if __name__ == "__main__":
    unittest.main()



import os
import sys
import unittest

import odil

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from peer_fixture_base import PeerFixtureBase

class TestMoveSCU(PeerFixtureBase):
    def setUp(self):
        PeerFixtureBase.setUp(
            self,
            [
                odil.AssociationParameters.PresentationContext(
                    1, odil.registry.PatientRootQueryRetrieveInformationModelMove,
                    [ odil.registry.ImplicitVRLittleEndian ], 
                    odil.AssociationParameters.PresentationContext.Role.SCU
                ),
                odil.AssociationParameters.PresentationContext(
                    3, odil.registry.RawDataStorage,
                    [ odil.registry.ImplicitVRLittleEndian ], 
                    odil.AssociationParameters.PresentationContext.Role.SCP
                )
            ])

        self.query = odil.DataSet()
        self.query.add("PatientName", ["Doe^John"])
        self.query.add("QueryRetrieveLevel", ["PATIENT"])

    def test_default_constructor(self):
        move = odil.MoveSCU(self.association)
        self.assertEqual(move.get_move_destination(), "")
        self.assertEqual(move.get_incoming_port(), 0)

    def test_move_destination(self):
        move = odil.MoveSCU(self.association)
        move.set_move_destination("remote")
        self.assertEqual(move.get_move_destination(), "remote")

    def test_incoming_port(self):
        move = odil.MoveSCU(self.association)
        move.set_incoming_port(11113)
        self.assertEqual(move.get_incoming_port(), 11113)

    def test_without_callback(self):
        move = odil.MoveSCU(self.association)
        move.set_affected_sop_class(odil.registry.PatientRootQueryRetrieveInformationModelMove)
        move.set_move_destination("LOCAL")
        move.set_incoming_port(11113)
        data_sets = move.move(self.query)

        self.assertEqual(len(data_sets), 1)
        self.assertSequenceEqual(
            data_sets[0].as_string("SOPInstanceUID"),
            [b"2.25.95090344942250266709587559073467305647"])

    def test_both_callback(self):
        data_sets = []
        messages = []
        def store_callback(data_set):
            data_sets.append(data_set)
        def move_callback(message):
            messages.append(message)

        move = odil.MoveSCU(self.association)
        move.set_affected_sop_class(odil.registry.PatientRootQueryRetrieveInformationModelMove)
        move.set_move_destination("LOCAL")
        move.set_incoming_port(11113)
        move.move(self.query, store_callback, move_callback)

        self.assertEqual(len(data_sets), 1)
        self.assertSequenceEqual(
            data_sets[0].as_string("SOPInstanceUID"),
            [b"2.25.95090344942250266709587559073467305647"])

        self.assertEqual(len(messages), 2)

        self.assertEqual(
            messages[0].get_number_of_remaining_sub_operations(), 0)
        self.assertEqual(
            messages[0].get_number_of_completed_sub_operations(), 1)
        self.assertEqual(messages[0].get_number_of_failed_sub_operations(), 0)
        self.assertEqual(messages[0].get_number_of_warning_sub_operations(), 0)

        self.assertFalse(messages[1].has_number_of_remaining_sub_operations())
        self.assertEqual(
            messages[1].has_number_of_completed_sub_operations(), 1)
        self.assertEqual(
            messages[1].get_number_of_failed_sub_operations(), 0)
        self.assertEqual(
            messages[1].get_number_of_warning_sub_operations(), 0)

    def test_store_only_callback(self):
        data_sets = []
        def store_callback(data_set):
            data_sets.append(data_set)

        move = odil.MoveSCU(self.association)
        move.set_affected_sop_class(odil.registry.PatientRootQueryRetrieveInformationModelMove)
        move.set_move_destination("LOCAL")
        move.set_incoming_port(11113)
        move.move(self.query, store_callback, None)

        self.assertEqual(len(data_sets), 1)
        self.assertSequenceEqual(
            data_sets[0].as_string("SOPInstanceUID"),
            [b"2.25.95090344942250266709587559073467305647"])

if __name__ == "__main__":
    unittest.main()



import pickle
import unittest

import odil

class TestVR(unittest.TestCase):
    def test_bytes_constructor(self):
        vr = odil.VR(b"UT")
        self.assertEqual(vr, odil.VR.UT)

    def test_unicode_constructor(self):
        vr = odil.VR(u"UT")
        self.assertEqual(vr, odil.VR.UT)

    def test_invalid_bytes_constructor(self):
        with self.assertRaises(Exception):
            odil.VR(b"XX")

    def test_invalid_unicode_constructor(self):
        with self.assertRaises(Exception):
            odil.VR(u"XX")

    def test_string(self):
        vr = odil.VR.AE
        self.assertEqual(str(vr), "AE")
    
    def test_pickle(self):
        self.assertEqual(pickle.loads(pickle.dumps(odil.VR.AE)), odil.VR.AE)

if __name__ == "__main__":
    unittest.main()



import os
import shutil
import tempfile
import unittest

import odil

class TestBasicDirectoryCreator(unittest.TestCase):
    def setUp(self):
        self.root = tempfile.mkdtemp()

        data_set_1 = odil.DataSet()
        data_set_1.add("PatientID", ["DJ123"])
        data_set_1.add("StudyDate", ["19100110"])
        data_set_1.add("StudyTime", ["1234"])
        data_set_1.add("StudyDescription", ["Study"])
        data_set_1.add("StudyInstanceUID", ["1.2.3.4"])
        data_set_1.add("StudyID", ["FOO"])
        data_set_1.add("Modality", ["OT"])
        data_set_1.add("SeriesInstanceUID", ["1.2.3.4.1"])
        data_set_1.add("SeriesNumber", [1])
        data_set_1.add("SeriesDescription", ["Series"])
        data_set_1.add("InstanceNumber", [1])
        data_set_1.add("SOPInstanceUID", ["1.2.3.4.1.1"])
        data_set_1.add("SOPClassUID", [odil.registry.RawDataStorage])

        with open(os.path.join(self.root, "a.dcm"), "wb") as fd:
            stream_1 = odil.iostream(fd)
            odil.Writer.write_file(data_set_1, stream_1)

        data_set_2 = odil.DataSet()
        data_set_2.add("PatientID", ["DJ123"])
        data_set_2.add("StudyDate", ["19100110"])
        data_set_2.add("StudyTime", ["1234"])
        data_set_2.add("StudyDescription", ["Study"])
        data_set_2.add("StudyInstanceUID", ["1.2.3.4"])
        data_set_2.add("StudyID", ["FOO"])
        data_set_2.add("Modality", ["OT"])
        data_set_2.add("SeriesInstanceUID", ["1.2.3.4.1"])
        data_set_2.add("SeriesNumber", [1])
        data_set_2.add("SeriesDescription", ["Series"])
        data_set_2.add("InstanceNumber", [2])
        data_set_2.add("SOPInstanceUID", ["1.2.3.4.1.2"])
        data_set_2.add("SOPClassUID", [odil.registry.RawDataStorage])

        with open(os.path.join(self.root, "b.dcm"), "wb") as fd:
            stream_2 = odil.iostream(fd)
            odil.Writer.write_file(data_set_2, stream_2)

        self.files = ["a.dcm", "b.dcm"]

    def tearDown(self):
        shutil.rmtree(self.root)

    def test_default_record_keys(self):
        creator = odil.BasicDirectoryCreator(self.root, self.files)
        creator()
        with open(os.path.join(self.root, "DICOMDIR"), "rb") as fd:
            header, dicomdir = odil.Reader.read_file(odil.iostream(fd))

        records = dicomdir.as_data_set("DirectoryRecordSequence")

        self._test_default_records(records)

    def test_extra_record_keys_list(self):
        creator = odil.BasicDirectoryCreator(
            self.root, self.files,
            { "SERIES": [["SeriesDescription", 1]]})
        creator()
        with open(os.path.join(self.root, "DICOMDIR"), "rb") as fd:
            header, dicomdir = odil.Reader.read_file(odil.iostream(fd))

        records = dicomdir.as_data_set("DirectoryRecordSequence")

        self._test_default_records(records)
        self.assertSequenceEqual(
            records[2].as_string("SeriesDescription"), [b"Series"])

    def test_extra_record_keys_tuple(self):
        creator = odil.BasicDirectoryCreator(
            self.root, self.files,
            { "SERIES": (("SeriesDescription", 1), )})
        creator()
        with open(os.path.join(self.root, "DICOMDIR"), "rb") as fd:
            header, dicomdir = odil.Reader.read_file(odil.iostream(fd))

        records = dicomdir.as_data_set("DirectoryRecordSequence")

        self._test_default_records(records)
        self.assertSequenceEqual(
            records[2].as_string("SeriesDescription"), [b"Series"])

    def _test_default_records(self, records):
        self.assertEqual(len(records), 5)

        self.assertSequenceEqual(
            records[0].as_string("DirectoryRecordType"), [b"PATIENT"])
        self.assertSequenceEqual(
            records[0].as_string("PatientID"), [b"DJ123"])

        self.assertSequenceEqual(
            records[1].as_string("DirectoryRecordType"), [b"STUDY"])
        self.assertSequenceEqual(
            records[1].as_string("StudyInstanceUID"), [b"1.2.3.4"])
        self.assertSequenceEqual(
            records[1].as_string("StudyDescription"), [b"Study"])

        self.assertSequenceEqual(
            records[2].as_string("DirectoryRecordType"), [b"SERIES"])
        self.assertSequenceEqual(
            records[2].as_string("SeriesInstanceUID"), [b"1.2.3.4.1"])

        self.assertSequenceEqual(
            records[3].as_string("DirectoryRecordType"), [b"IMAGE"])
        self.assertSequenceEqual(
            records[3].as_int("InstanceNumber"), [1])

        self.assertSequenceEqual(
            records[4].as_string("DirectoryRecordType") , [b"IMAGE"])
        self.assertSequenceEqual(
            records[4].as_int("InstanceNumber"), [2])

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestAssociation(unittest.TestCase):
    def test_default_constructor(self):
        association = odil.Association()
        self.assertFalse(association.is_associated())

    def test_peer_host(self):
        association = odil.Association()
        association.set_peer_host("pacs.example.com")
        self.assertEqual(association.get_peer_host(), "pacs.example.com")

    def test_peer_port(self):
        association = odil.Association()
        association.set_peer_port(1234)
        self.assertEqual(association.get_peer_port(), 1234)

    def test_peer_port(self):
        association = odil.Association()
        association.set_peer_port(1234)
        self.assertEqual(association.get_peer_port(), 1234)

    def test_parameters(self):
        parameters = odil.AssociationParameters()
        parameters.set_called_ae_title("foo")

        association = odil.Association()
        association.set_parameters(parameters)
        self.assertEqual(
            association.get_parameters().get_called_ae_title(), "foo")

    def test_update_parameters(self):
        association = odil.Association()
        association.update_parameters().set_called_ae_title("foo")
        self.assertEqual(
            association.get_parameters().get_called_ae_title(), "foo")

    def test_next_message_id(self):
        association = odil.Association()
        id1 = association.next_message_id()
        id2 = association.next_message_id()
        self.assertNotEqual(id1, id2)

if __name__ == "__main__":
    unittest.main()




import unittest

import odil

class TestUID(unittest.TestCase):
    def test_uid_prefix(self):
        self.assertEqual(odil.uid_prefix, "1.2.826.0.1.3680043.9.5560")

    def test_uid_prefix(self):
        uid = odil.generate_uid()
        self.assertEqual(uid[:len(odil.uid_prefix)], odil.uid_prefix)

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestHTTPRequest(unittest.TestCase):
    def test_default_constructor(self):
        request = odil.webservices.HTTPRequest()
        self.assertEqual(request.get_method(), "")
        self.assertEqual(str(request.get_target()), "")
        self.assertEqual(request.get_http_version(), "HTTP/1.0")
        self.assertEqual(dict(request.get_headers()), {})
        self.assertEqual(request.get_body(), b"")

    def test_full_constructor(self):
        request = odil.webservices.HTTPRequest(
            method="GET",
            target=odil.webservices.URL.parse("http://example.com/foo?bar=quux"),
            http_version="HTTP/1.1", headers={"foo": "bar"}, body="body")
        self.assertEqual(request.get_method(), "GET")
        self.assertEqual(
            str(request.get_target()), "http://example.com/foo?bar=quux")
        self.assertEqual(request.get_http_version(), "HTTP/1.1")
        self.assertEqual(dict(request.get_headers()), {"foo":"bar"})
        self.assertEqual(request.get_body(), b"body")

    def test_method(self):
        request = odil.webservices.HTTPRequest()
        request.set_method("POST")
        self.assertEqual(request.get_method(), "POST")

    def test_target(self):
        request = odil.webservices.HTTPRequest()
        request.set_target(
            odil.webservices.URL.parse("http://example.com/foo?bar=quux"))
        self.assertEqual(
            str(request.get_target()), "http://example.com/foo?bar=quux")

    def test_http_version(self):
        request = odil.webservices.HTTPRequest()
        request.set_http_version("HTTP/1.1")
        self.assertEqual(request.get_http_version(), "HTTP/1.1")

    def test_headers(self):
        request = odil.webservices.HTTPRequest()
        request.set_headers({"foo": "bar"})
        self.assertEqual(dict(request.get_headers()), {"foo": "bar"})

    def test_body(self):
        request = odil.webservices.HTTPRequest()
        request.set_body("body")
        self.assertEqual(request.get_body(), b"body")

if __name__ == "__main__":
    unittest.main()



import unittest

import odil

class TestAssociationParameters(unittest.TestCase):
    def test_default_constructor(self):
        parameters = odil.AssociationParameters()
        self.assertEqual(parameters.get_called_ae_title(), "")
        self.assertEqual(parameters.get_calling_ae_title(), "")
        self.assertEqual(len(parameters.get_presentation_contexts()), 0)

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            getattr(odil.AssociationParameters.UserIdentity.Type, "None"))

        self.assertEqual(parameters.get_maximum_length(), 16384)

    def test_called_ae_title(self):
        parameters = odil.AssociationParameters()
        parameters.set_called_ae_title("foo")
        self.assertEqual(parameters.get_called_ae_title(), "foo")

    def test_calling_ae_title(self):
        parameters = odil.AssociationParameters()
        parameters.set_calling_ae_title("foo")
        self.assertEqual(parameters.get_calling_ae_title(), "foo")

    def test_presentation_contexts(self):
        presentation_context = odil.AssociationParameters.PresentationContext(
            1, "foo", ["bar"], 
            odil.AssociationParameters.PresentationContext.Role.SCU)
        parameters = odil.AssociationParameters()
        parameters.set_presentation_contexts([presentation_context])

        self.assertEqual(len(parameters.get_presentation_contexts()), 1)
        self.assertEqual(
            parameters.get_presentation_contexts()[0], presentation_context)

    def test_presentation_contexts_simplified(self):
        presentation_context = odil.AssociationParameters.PresentationContext(
            "quux", ["plip", "plop"], 
            odil.AssociationParameters.PresentationContext.Role.SCU)
        parameters = odil.AssociationParameters()
        parameters.set_presentation_contexts([presentation_context])

        self.assertEqual(len(parameters.get_presentation_contexts()), 1)
        # ID is modified since not user-supplied
        presentation_context.id = parameters.get_presentation_contexts()[0].id
        self.assertEqual(
            parameters.get_presentation_contexts()[0], presentation_context)

    def test_user_identity_username(self):
        parameters = odil.AssociationParameters()
        parameters.set_user_identity_to_username("foo")

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            odil.AssociationParameters.UserIdentity.Type.Username)
        self.assertEqual(user_identity.primary_field, "foo")

    def test_user_identity_username_and_password(self):
        parameters = odil.AssociationParameters()
        parameters.set_user_identity_to_username_and_password("foo", "bar")

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            odil.AssociationParameters.UserIdentity.Type.UsernameAndPassword)
        self.assertEqual(user_identity.primary_field, "foo")
        self.assertEqual(user_identity.secondary_field, "bar")

    def test_user_identity_kerberos(self):
        parameters = odil.AssociationParameters()
        parameters.set_user_identity_to_kerberos("foo")

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            odil.AssociationParameters.UserIdentity.Type.Kerberos)
        self.assertEqual(user_identity.primary_field, "foo")

    def test_user_identity_saml(self):
        parameters = odil.AssociationParameters()
        parameters.set_user_identity_to_saml("foo")

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            odil.AssociationParameters.UserIdentity.Type.SAML)
        self.assertEqual(user_identity.primary_field, "foo")

    def test_user_identity_none(self):
        parameters = odil.AssociationParameters()
        parameters.set_user_identity_to_saml("foo")
        parameters.set_user_identity_to_none()

        user_identity = parameters.get_user_identity()
        self.assertEqual(
            user_identity.type, 
            getattr(odil.AssociationParameters.UserIdentity.Type, "None"))

    def test_maximum_length(self):
        parameters = odil.AssociationParameters()
        parameters.set_maximum_length(12345)
        self.assertEqual(parameters.get_maximum_length(), 12345)

    def test_chaining(self):
        parameters = odil.AssociationParameters()
        parameters.set_called_ae_title("foo").set_calling_ae_title("bar")
        self.assertEqual(parameters.get_called_ae_title(), "foo")
        self.assertEqual(parameters.get_calling_ae_title(), "bar")
    
class TestPresentationContext(unittest.TestCase):
    def test_constructor(self):
        presentation_context = odil.AssociationParameters.PresentationContext(
            1, "foo", ["bar", "baz"], 
            odil.AssociationParameters.PresentationContext.Role.SCP)
        self.assertEqual(presentation_context.id, 1)
        self.assertEqual(presentation_context.abstract_syntax, "foo")
        self.assertEqual(
            [x for x in presentation_context.transfer_syntaxes], [b"bar", b"baz"])
        self.assertEqual(
            presentation_context.role, 
            odil.AssociationParameters.PresentationContext.Role.SCP)

if __name__ == "__main__":
    unittest.main()



import os
import unittest

import odil

class PeerFixtureBase(unittest.TestCase):
    def setUp(self, contexts):
        self.association = odil.Association()
        self.association.set_peer_host(os.environ["ODIL_PEER_HOST_NAME"])
        self.association.set_peer_port(int(os.environ["ODIL_PEER_PORT"]))
        self.association.update_parameters()\
            .set_calling_ae_title(os.environ["ODIL_OWN_AET"])\
            .set_called_ae_title(os.environ["ODIL_PEER_AET"]) \
            .set_presentation_contexts(contexts)
        self.association.associate()

    def tearDown(self):
        self.association.release()
